#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Dan Smith <danms@us.ibm.com>
# Author: Ryan Harper <ryanh@us.ibm.com>

from XmTestLib import *

def get_sedf_params(domain):
    status, output = traceCommand("xm sched-sedf %s" %(domain.getName()))
    return (status, output.split('\n')[1].split())


domain = XmTestDomain(extraConfig = {"sched":"sedf"})

try:
    domain.start(noConsole=True)
except DomainError, e:
    if verbose:
        print "Failed to create test domain because:"
        print e.extra
    FAIL(str(e))

# get current param values as baseline
(status, params) = get_sedf_params(domain)

# check rv
if status != 0:
    FAIL("Getting sedf parameters return non-zero rv (%d)", status)

# parse out current params
(name, domid, p, s, l, e, w) = params

# toggle extratime value
extratime = str((int(e)+1)%2)

direction = "disable"
# NB: when disabling extratime(=0), must pass in a slice
opts = "%s -e %s" %(domain.getName(), extratime)
if extratime == "0":
    opts += " -s %s" %( str( (float(p)/2)+1 ) )
    direction = "enable"
    
(status, output) = traceCommand("xm sched-sedf %s" %(opts))

# check rv
if status != 0:
   FAIL("Setting sedf parameters return non-zero rv (%d)" % status)

# validate
(s,params) = get_sedf_params(domain)

# check rv
if s != 0:
    FAIL("Getting sedf parameters return non-zero rv (%d)" % s)

(name,domid,p1,s1,l1,e1,w1) = params

if e1 != extratime:
    FAIL("Failed to %s extratime" %(direction))

# Stop the domain (nice shutdown)
domain.stop()



#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Dan Smith <danms@us.ibm.com>

#
# 10 start a domain
# 20 destroy it
# 30 try next xm command
# 40 goto 10

from XmTestLib import *

import re

def test_mem_set(name):
    status, output = traceCommand("xm mem-set %s 32" % name, logOutput=True)

    if status == 0:
        FAIL("mem-set worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("mem-set failed to report error after destroy!")

def test_pause(name):
    status, output = traceCommand("xm pause %s" % name, logOutput=True)

    if status == 0:
        FAIL("pause worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("pause failed to report error after destroy!")

def test_unpause(name):
    status, output = traceCommand("xm unpause %s" % name, logOutput=True)

    if status == 0:
        FAIL("unpause worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("unpause failed to report error after destroy!")

def test_reboot(name):
    status, output = traceCommand("xm reboot %s" % name, logOutput=True)

    if status == 0:
        FAIL("reboot worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("reboot failed to report error after destroy!")

def test_save(name):
    status, output = traceCommand("xm save %s /tmp/foo" % name, logOutput=True)

    if status == 0:
        FAIL("save worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("save failed to report error after destroy!")

def test_block_list(name):
    status, output = traceCommand("xm block-list %s" % name, logOutput=True)

    if status == 0:
        FAIL("block-list worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("block-list failed to report error after destroy!")

def test_shutdown(name):
    status, output = traceCommand("xm shutdown %s" % name, logOutput=True)

    if status == 0:
        FAIL("shutdown worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("shutdown failed to report error after destroy!")

def test_domname(name):
    status, output = traceCommand("xm domname %s" % name)

    if status == 0:
        FAIL("domname worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("domname failed to report error after destroy!")

def test_domid(name):
    status, output = traceCommand("xm domid %s" % name)

    if status == 0:
        FAIL("domid worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("domid failed to report error after destroy!")

def test_destroy(name):
    status, output = traceCommand("xm destroy %s" % name)

    if status == 0:
        FAIL("destroy worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("destroy failed to report error after destroy!")

def test_sysrq(name):
    status, output = traceCommand("xm sysrq %s s" % name)

    if status == 0:
        FAIL("sysrq worked after domain destroy!")
    if not re.search("[Ee]rror", output):
        FAIL("sysrq failed to report error after destroy!")

def runTests(tests):
    for test in tests:
        domain = XmTestDomain()

        # Create a domain

        try:
            console = domain.start()
        except DomainError, e:
            FAIL(str(e))

        try:
            console.runCmd("ls")
        except ConsoleError, e:
            FAIL(str(e))

        # Destroy it
                
        domain.destroy()
        
        # Run test
        
        test(domain.getName())

tests = [test_mem_set, test_pause, test_unpause, test_reboot, test_save,
         test_block_list, test_shutdown, test_domid, test_domname]

if verbose:
    print "Running stale tests"
runTests(tests)



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2005 Mike Wray <mike.wray@hp.com>
# Copyright (C) 2005-2006 XenSource Ltd.
#============================================================================


import os
import os.path
import socket
import stat

from xen.util import mkdir

import connection


def bind(path, type = socket.SOCK_STREAM):
    """Create a Unix socket, and bind it to the given path.
    The socket is created such that only the current user may access it."""

    if path[0] == '\0': # Abstract namespace is used for the path
        pass
    else:
        parent = os.path.dirname(path)
        mkdir.parents(parent, stat.S_IRWXU, True)
        if os.path.exists(path):
            os.unlink(path)

    sock = socket.socket(socket.AF_UNIX, type)
    sock.bind(path)
    return sock


class UnixListener(connection.SocketListener):
    def __init__(self, path, protocol_class):
        self.path = path
        connection.SocketListener.__init__(self, protocol_class)


    def createSocket(self):
        return bind(self.path, socket.SOCK_STREAM)


    def acceptConnection(self, sock, _):
        connection.SocketServerConnection(sock, self.protocol_class)


class UnixDgramListener(connection.SocketDgramListener):
    def __init__(self, path, protocol_class):
        self.path = path
        connection.SocketDgramListener.__init__(self, protocol_class)


    def createSocket(self):
        return bind(self.path, socket.SOCK_DGRAM)




#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2009 flonatel GmbH & Co. KG
#============================================================================

import os
import unittest

from xen.util.fileuri import scheme_error
from xen.util.fileuri import scheme_data
from xen.util.fileuri import scheme_file
from xen.util.fileuri import schemes

class scheme_data_unit_tests(unittest.TestCase):

    def check_basic_encoding(self):
        "util.fileuri.scheme_data - basic encoding"
        sd = scheme_data.encode('Hello!')
        self.assertEqual(sd, 'data:application/octet-stream;base64,SGVsbG8h')

    def check_encoding_with_given_mediatype(self):
        "util.fileuri.scheme_data - encoding with given media name"
        sd = scheme_data.encode('Hello!', 'application/x-my-linux-kernel')
        self.assertEqual(sd,
              'data:application/x-my-linux-kernel;base64,SGVsbG8h')

    def check_parse_01(self):
        "util.fileuri.scheme_data - parsing of None"
        self.assertRaises(scheme_error, scheme_data.parse, None)

    def check_parse_02(self):
        "util.fileuri.scheme_data - parsing of empty string"
        self.assertRaises(scheme_error, scheme_data.parse, "")

    def check_parse_03(self):
        "util.fileuri.scheme_data - parsing of unstructured data"
        self.assertRaises(scheme_error, scheme_data.parse, "akskdjdfhezezu")

    def check_parse_04(self):
        "util.fileuri.scheme_data - data: is not at the first place"
        self.assertRaises(scheme_error, scheme_data.parse, 'ggdata:sossm')

    def check_parse_05(self):
        "util.fileuri.scheme_data - no comma in data"
        self.assertRaises(scheme_error, scheme_data.parse, 'data:sossm')

    def check_parse_06(self):
        "util.fileuri.scheme_data - encoding is empty"
        self.assertRaises(scheme_error, scheme_data.parse, 'data:,')

    def check_parse_07(self):
        "util.fileuri.scheme_data - unknown encoding"
        self.assertRaises(scheme_error, scheme_data.parse,
                          'data:somemediatype;unknown,')

    def check_parse_08(self):
        "util.fileuri.scheme_data - parse ok - empty data"
        mediatype, encoding, data_start = scheme_data.parse(
            'data:somemedia;base64,')
        self.assertEqual(mediatype, 'somemedia')
        self.assertEqual(encoding, 'base64')
        self.assertEqual(data_start, 22)

    def check_parse_09(self):
        "util.fileuri.scheme_data - parse ok - some data"
        mediatype, encoding, data_start = scheme_data.parse(
            'data:somemedia;base64,HereComesTheSun')
        self.assertEqual(mediatype, 'somemedia')
        self.assertEqual(encoding, 'base64')
        self.assertEqual(data_start, 22)

    def check_parse_10(self):
        "util.fileuri.scheme_data - header ok - data error"
        self.assertRaises(scheme_error, scheme_data.decode,
               'data:application/octet-stream;base64,H!$ere"Co<mesT>heS_.un')

    def check_cff_file_does_not_exist(self):
        "util.fileuri.scheme_data - create from file - non existent file"
        self.assertRaises(scheme_error, scheme_data.create_from_file,
                          "/there/is/hopefully/no/file/like/this")

    def check_cff_ok(self):
        "util.fileuri.scheme_data - create from file - ok"
        tmppath = "/tmp/scheme_data_check_cff_ok"
        f = open(tmppath, "w")
        f.write("huhuhu")
        f.close()
        d = scheme_data.create_from_file(tmppath)
        os.unlink(tmppath)
        self.assertEqual(d, "data:application/octet-stream;base64,aHVodWh1")


class scheme_file_unit_tests(unittest.TestCase):

    def check_encode_empty_filename(self):
        "util.fileuri.scheme_file - encode empty filename"
        self.assertRaises(scheme_error, scheme_file.encode, "")

    def check_encode_relative_filename(self):
        "util.fileuri.scheme_file - encode relative filename"
        self.assertRaises(scheme_error, scheme_file.encode, "../there")

    def check_encode_absolut_filename(self):
        "util.fileuri.scheme_file - encode absolut filename"
        self.assertEqual(
            scheme_file.encode("/here/and/there/again"),
            'file:///here/and/there/again')

    def check_decode_01(self):
        "util.fileuri.scheme_file - decode empty data"
        self.assertRaises(scheme_error, scheme_file.decode, "")

    def check_decode_02(self):
        "util.fileuri.scheme_file - decode data with no file:// at the beginning (1)"
        self.assertRaises(scheme_error, scheme_file.decode,
                          "phonehome://bbbb")

    def check_decode_03(self):
        "util.fileuri.scheme_file - decode data with no file:// at the beginning (2)"
        self.assertRaises(scheme_error, scheme_file.decode,
                          "file:/bbbb")

    def check_decode_04(self):
        "util.fileuri.scheme_file - decode empty path"
        self.assertRaises(scheme_error, scheme_file.decode,
                          "file://")

    def check_decode_05(self):
        "util.fileuri.scheme_file - decode empty relative path"
        self.assertRaises(scheme_error, scheme_file.decode,
                          "file://somewhere")

    def check_decode_06(self):
        "util.fileuri.scheme_file - decode ok"
        path, tmp_file = scheme_file.decode("file:///boot/vmlinuz")
        self.assertEqual(path, "/boot/vmlinuz")
        self.assertEqual(tmp_file, False)

class scheme_set_unit_tests(unittest.TestCase):

    def check_data_01(self):
        "util.fileuri.scheme_set - data with error in media type"

        u = "data:something_wrong,base64:swer"
        uri, tmp_file = schemes.decode(u)
        self.assertEqual(uri, u)
        self.assertEqual(tmp_file, False)

    def check_data_02(self):
        "util.fileuri.scheme_set - data with error in base64 data"

        u = "data:application/octet-stream;base64,S!VsbG8h"
        uri, tmp_file = schemes.decode(u)
        self.assertEqual(uri, u)
        self.assertEqual(tmp_file, False)
 
    def check_data_03(self):
        "util.fileuri.scheme_set - data ok"

        u = "data:application/octet-stream;base64,SGVsbG8h"
        uri, tmp_file = schemes.decode(u)

        # Read file contents
        f = open(uri, "r")
        d = f.read()
        f.close()
        os.unlink(uri)

        self.assertEqual(d, "Hello!")
        self.assertEqual(tmp_file, True)
       
    def check_file_01(self):
        "util.fileuri.scheme_set - file ok"

        f = "/The/Path/To/The/File.txt"
        uri, tmp_file = schemes.decode("file://" + f)
        self.assertEqual(uri, f)
        self.assertEqual(tmp_file, False)

    def check_without_scheme_01(self):
        "util.fileuri.scheme_set - without scheme"

        f = "/The/Path/To/The/File.txt"
        uri, tmp_file = schemes.decode(f)
        self.assertEqual(uri, f)
        self.assertEqual(tmp_file, False)


def suite():
    return unittest.TestSuite(
        [unittest.makeSuite(scheme_data_unit_tests, 'check_'),
         unittest.makeSuite(scheme_file_unit_tests, 'check_'),
         unittest.makeSuite(scheme_set_unit_tests, 'check_'),])

if __name__ == "__main__":
    testresult = unittest.TextTestRunner(verbosity=3).run(suite())




#!/usr/bin/python
#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2007 Tom Wilkie <tom.wilkie@gmail.com>
# Copyright (C) 2010 ANEXIA Internetdienstleistungs GmbH
#   Author: Stephan Peijnik <spe@anexia.at>
#============================================================================
"""Domain creation using new XenAPI
"""

from xen.xm.main import server, get_default_SR
from xml.dom.minidom import parse, getDOMImplementation
from lxml import etree
from xen.xend import sxp
from xen.xend.XendAPIConstants import XEN_API_ON_NORMAL_EXIT, \
     XEN_API_ON_CRASH_BEHAVIOUR
from xen.xm.opts import OptionError
from xen.util import xsconstants
from xen.util.pci import pci_opts_list_from_sxp
from xen.util.path import SHAREDIR
import xen.util.xsm.xsm as security

import sys
import os
from os.path import join
import traceback
import re
import warnings # Used by lxml-based validator

def log(_, msg):
    #print "> " + msg
    pass

DEBUG = 0

def get_name_label(node):
    name_node = node.getElementsByTagName("name")[0]
    label_node = name_node.getElementsByTagName("label")[0]
    return " ".join([child.nodeValue for child in label_node.childNodes])

def get_name_description(node):
    name_node = node.getElementsByTagName("name")[0]
    description_node = name_node.getElementsByTagName("description")[0]
    return " ".join([child.nodeValue for child in description_node.childNodes])

def get_text_in_child_node(node, child):
    tag_node = node.getElementsByTagName(child)[0]
    return " ".join([child.nodeValue for child in tag_node.childNodes])

def get_child_node_attribute(node, child, attribute):
    tag_node = node.getElementsByTagName(child)[0]
    return tag_node.attributes[attribute].value

def get_child_nodes_as_dict(node, child_name,
                            key_attribute_name,
                            value_attribute_name):
    return dict([(child.attributes[key_attribute_name].value,
                  child.attributes[value_attribute_name].value)
                 for child in node.getElementsByTagName(child_name)])

def try_quietly(fn, *args):
    try:
        return fn(*args)
    except:
        return None

class xenapi_create:

    def __init__(self):
        self.DEFAULT_STORAGE_REPOSITORY = get_default_SR()

        self.dtd = join(SHAREDIR, "xen", "create.dtd")

    def create(self, filename=None, document=None, skipdtd=False):
        """
        Create a domain from an XML file or DOM tree
        """
        if skipdtd:
            print "Skipping DTD checks.  Dangerous!"
        
        if filename is not None:
            if not skipdtd:
                self.check_dtd(filename)
            document = parse(filename)
        elif document is not None:
            if not skipdtd:
                self.check_dom_against_dtd(document)

        self.check_doc(document)

        vdis = document.getElementsByTagName("vdi")
        vdi_refs_dict = self.create_vdis(vdis)

        networks = document.getElementsByTagName("network")
        network_refs_dict = self.create_networks(networks)
        
        try:    
            vms = document.getElementsByTagName("vm")
            return self.create_vms(vms, vdi_refs_dict, network_refs_dict)
        except Exception, exn:
            try_quietly(self.cleanup_vdis(vdi_refs_dict))
            raise exn

    # Methods to check xml file
    # try to use dtd to check where possible
    def check_dtd(self, file):
        """
        Check file against DTD.
        Use this if possible as it gives nice
        error messages
        """
        try:
            dtd = etree.DTD(open(self.dtd, 'r'))
        except IOError:
            # The old code did neither raise an exception here, nor
            # did it report an error. For now we issue a warning.
            # TODO: How to handle a missing dtd file?
            # --sp
            warnings.warn('DTD file %s not found.' % (self.dtd),
                          UserWarning)
            return
        
        tree = etree.parse(file)
        root = tree.getroot()
        if not dtd.validate(root):
            self.handle_dtd_errors(dtd)
            
    def check_dom_against_dtd(self, dom):
        """
        Check DOM again DTD.
        Doesn't give as nice error messages.
        (no location info)
        """
        try:
            dtd = etree.DTD(open(self.dtd, 'r'))
        except IOError:
            # The old code did neither raise an exception here, nor
            # did it report an error. For now we issue a warning.
            # TODO: How to handle a missing dtd file?
            # --sp
            warnings.warn('DTD file %s not found.' % (self.dtd),
                          UserWarning)
            return

        # XXX: This may be a bit slow. Maybe we should use another way
        # of getting an etree root element from the minidom DOM tree...
        # -- sp
        root = etree.XML(dom.toxml())
        if not dtd.validate(root):
            self.handle_dtd_errors(dtd)

    # Do the same that was done in report_error before. This is directly
    # called by check_dtd and check_dom_against_dtd.
    # We are using sys.stderr instead of print though (python3k clean).
    def handle_dtd_errors(self, dtd):
        # XXX: Do we really want to bail out here?
        # -- sp
        for err in dtd.error_log:
            err_str = 'ERROR: %s\n' % (str(err),)
            sys.stderr.write(err_str)
        sys.stderr.flush()
        sys.exit(-1)

    #
    # Checks which cannot be done with dtd
    #
    def check_doc(self, doc):
        vms = doc.getElementsByTagName("vm")
        self.check_vms(vms)

    def check_vms(self, vms):
        map(self.check_vm, vms)

    def check_vm(self, vm):
        vifs = vm.getElementsByTagName("vif")
        self.check_vifs(vifs)

    def check_vifs(self, vifs):
        map(self.check_vif, vifs)

    def check_vif(self, vif):
        pass

    # Cleanup methods here
    def cleanup_vdis(self, vdi_refs_dict):
        map(self.cleanup_vdi, vdi_refs_dict.values())

    def cleanup_vdi(self, vdi_ref):
        server.xenapi.VDI.destroy(vdi_ref)

    def cleanup_vms(self, vm_refs):
        map(self.cleanup_vm, vm_refs)

    def cleanup_vm(self, vm_ref):
        server.xenapi.VM.destroy(vm_ref)

    # Create methods here
    def create_vdis(self, vdis):
        log(DEBUG, "create_vdis")
        return dict(map(self.create_vdi, vdis))

    def create_vdi(self, vdi):
        log(DEBUG, "create_vdi")

        for ref, record in server.xenapi.VDI.get_all_records().items():
            location = record["other_config"]["location"]
            if vdi.attributes["src"].value != location:
                continue

            # Reuse the VDI because the location is same.
            key = vdi.attributes["name"].value
            return (key, ref)

        # Create a new VDI.
        vdi_record = {
            "name_label":       get_name_label(vdi),
            "name_description": get_name_description(vdi),
            "SR":               self.DEFAULT_STORAGE_REPOSITORY,  
            "virtual_size":     vdi.attributes["size"].value,
            "type":             vdi.attributes["type"].value,
            "sharable":         vdi.attributes["sharable"].value == "True",
            "read_only":        vdi.attributes["read_only"].value == "True",
            "other_config":     {"location":
                vdi.attributes["src"].value}
            }

        key = vdi.attributes["name"].value
        value = server.xenapi.VDI.create(vdi_record)
        
        return (key, value)

    def create_networks(self, networks):
        log(DEBUG, "create_networks")
        return dict(map(self.create_network, networks))

    def create_network(self, network):
        log(DEBUG, "create_network")

        network_record = {
            "name_label":       get_name_label(network),
            "name_description": get_name_description(network),
            "other_config":
                get_child_nodes_as_dict(network, "other_config",
                                        "key", "value"),
            "default_netmask":  network.attributes["default_netmask"].value,
            "default_gateway":  network.attributes["default_gateway"].value
            }

        key = network.attributes["name"].value
        value = server.xenapi.network.create(network_record)

        return (key, value)
        
    def create_vms(self, vms, vdis, networks):
        log(DEBUG, "create_vms")
        return map(lambda vm: self.create_vm(vm, vdis, networks), vms)

    def create_vm(self, vm, vdis, networks):
        log(DEBUG, "create_vm")

        vm_record = {
            "name_label":
                get_name_label(vm),
            "name_description":
                get_name_description(vm),
            "user_version":
                get_text_in_child_node(vm, "version"),
            "is_a_template":
                vm.attributes["is_a_template"].value == 'true',
            "auto_power_on":
                vm.attributes["auto_power_on"].value == 'true',
            "s3_integrity":
                vm.attributes["s3_integrity"].value,
            "superpages":
                vm.attributes["superpages"].value,
            "memory_static_max":
                get_child_node_attribute(vm, "memory", "static_max"),
            "memory_static_min":
                get_child_node_attribute(vm, "memory", "static_min"),
            "memory_dynamic_max":
                get_child_node_attribute(vm, "memory", "dynamic_max"),
            "memory_dynamic_min":
                get_child_node_attribute(vm, "memory", "dynamic_min"),
            "VCPUs_params":
                get_child_nodes_as_dict(vm, "vcpu_param", "key", "value"),
            "VCPUs_max":
                vm.attributes["vcpus_max"].value,
            "VCPUs_at_startup":
                vm.attributes["vcpus_at_startup"].value,
            "actions_after_shutdown":
                vm.attributes["actions_after_shutdown"].value,
            "actions_after_reboot":
                vm.attributes["actions_after_reboot"].value,
            "actions_after_crash":
                vm.attributes["actions_after_crash"].value,
            "platform":
                get_child_nodes_as_dict(vm, "platform", "key", "value"),
            "other_config":
                get_child_nodes_as_dict(vm, "other_config", "key", "value"),
            "pool_name":
                vm.attributes["pool_name"].value,
            "PV_bootloader":
                "",
            "PV_kernel":
                "",
            "PV_ramdisk":
                "",
            "PV_args":
                "",
            "PV_bootloader_args":
                "",
            "HVM_boot_policy":
                "",
            "HVM_boot_params":
                {},
            "PCI_bus":
               ""
            }

        if vm.attributes.has_key("security_label"):
            vm_record.update({
                "security_label":
                    vm.attributes["security_label"].value
                })

        if len(vm.getElementsByTagName("pv")) > 0:
            vm_record.update({
                "PV_bootloader":
                    get_child_node_attribute(vm, "pv", "bootloader"),
                "PV_kernel":
                    get_child_node_attribute(vm, "pv", "kernel"),
                "PV_ramdisk":
                    get_child_node_attribute(vm, "pv", "ramdisk"),
                "PV_args":
                    get_child_node_attribute(vm, "pv", "args"),
                "PV_bootloader_args":
                    get_child_node_attribute(vm, "pv", "bootloader_args")
                })
        else:
            hvm = vm.getElementsByTagName("hvm")[0]
            vm_record.update({
                "HVM_boot_policy":
                    get_child_node_attribute(vm, "hvm", "boot_policy"),
                "HVM_boot_params":
                    get_child_nodes_as_dict(hvm, "boot_param", "key", "value")
                })
        try:
            vm_ref = server.xenapi.VM.create(vm_record)
        except:
            traceback.print_exc()
            sys.exit(-1)

        try:
            # Now create vbds

            vbds = vm.getElementsByTagName("vbd")

            self.create_vbds(vm_ref, vbds, vdis)

            # Now create vifs

            vifs = vm.getElementsByTagName("vif")

            self.create_vifs(vm_ref, vifs, networks)

            # Now create consoles

            consoles = vm.getElementsByTagName("console")

            self.create_consoles(vm_ref, consoles)

            # Now create pcis

            pcis = vm.getElementsByTagName("pci")

            self.create_pcis(vm_ref, pcis)

            # Now create scsis

            scsis = vm.getElementsByTagName("vscsi")

            self.create_scsis(vm_ref, scsis)

            return vm_ref
        except:
            server.xenapi.VM.destroy(vm_ref)
            raise
        
    def create_vbds(self, vm_ref, vbds, vdis):
        log(DEBUG, "create_vbds")
        return map(lambda vbd: self.create_vbd(vm_ref, vbd, vdis), vbds)

    def create_vbd(self, vm_ref, vbd, vdis):
        log(DEBUG, "create_vbd")

        vbd_record = {
            "VM":
                vm_ref,
            "VDI":
                vdis[vbd.attributes["vdi"].value],
            "device":
                vbd.attributes["device"].value,
            "bootable":
                vbd.attributes["bootable"].value == "1",
            "mode":
                vbd.attributes["mode"].value,
            "type":
                vbd.attributes["type"].value,
            "qos_algorithm_type":
                vbd.attributes["qos_algorithm_type"].value,
            "qos_algorithm_params":
                get_child_nodes_as_dict(vbd,
                  "qos_algorithm_param", "key", "value")
            }

        return server.xenapi.VBD.create(vbd_record)

    def create_vifs(self, vm_ref, vifs, networks):
        log(DEBUG, "create_vifs")
        return map(lambda vif: self.create_vif(vm_ref, vif, networks), vifs)

    def create_vif(self, vm_ref, vif, networks):
        log(DEBUG, "create_vif")

        if 'network' in vif.attributes.keys():
            network_name = vif.attributes['network'].value

            if network_name in networks.keys():
                network_uuid = networks[network_name]
            else:
                networks = dict([(record['name_label'], ref)
                                 for ref, record in
                                 server.xenapi.network.get_all_records().items()])
                if network_name in networks.keys():
                    network_uuid = networks[network_name]
                else:
                    raise OptionError("Network %s doesn't exist"
                                  % vif.attributes["network"].value)
        else:
            network_uuid = self._get_network_ref()

        vif_record = {
            "device":
                vif.attributes["device"].value,
            "network":
                network_uuid,
            "VM":
                vm_ref,
            "MAC":
                vif.attributes["mac"].value,
            "MTU":
                vif.attributes["mtu"].value,
            "qos_algorithm_type":
                vif.attributes["qos_algorithm_type"].value,
            "qos_algorithm_params":
                get_child_nodes_as_dict(vif,
                    "qos_algorithm_param", "key", "value"),
            "security_label":
                vif.attributes["security_label"].value
        }

        return server.xenapi.VIF.create(vif_record)

    _network_refs = []

    def _get_network_ref(self):
        try:
            return self._network_refs.pop(0)
        except IndexError:
            self._network_refs = server.xenapi.network.get_all()
            return self._network_refs.pop(0)

    def create_consoles(self, vm_ref, consoles):
        log(DEBUG, "create_consoles")
        return map(lambda console: self.create_console(vm_ref, console),
                   consoles)

    def create_console(self, vm_ref, console):
        log(DEBUG, "create_consoles")

        console_record = {
            "VM":
                vm_ref,
            "protocol":
                console.attributes["protocol"].value,
            "other_config":
                get_child_nodes_as_dict(console,
                  "other_config", "key", "value")
            }

        return server.xenapi.console.create(console_record)

    def create_pcis(self, vm_ref, pcis):
        log(DEBUG, "create_pcis")
        return map(lambda pci: self.create_pci(vm_ref, pci), pcis)

    def create_pci(self, vm_ref, pci):
        log(DEBUG, "create_pci")

        domain = int(pci.attributes["domain"].value, 16)
        bus = int(pci.attributes["bus"].value, 16)
        slot = int(pci.attributes["slot"].value, 16)
        func = int(pci.attributes["func"].value, 16)
        name = "%04x:%02x:%02x.%01x" % (domain, bus, slot, func)

        target_ref = None
        for ppci_ref in server.xenapi.PPCI.get_all():
            if name == server.xenapi.PPCI.get_name(ppci_ref):
                target_ref = ppci_ref
                break
        if target_ref is None:
            log(DEBUG, "create_pci: pci device not found")
            return None

        dpci_record = {
            "VM":
                vm_ref,
            "PPCI":
                target_ref,
            "hotplug_slot":
                int(pci.attributes["vdevfn"].value, 16),
            "options":
                get_child_nodes_as_dict(pci,
                  "pci_opt", "key", "value"),
            "key":
                pci.attributes["key"].value
        }

        return server.xenapi.DPCI.create(dpci_record)

    def create_scsis(self, vm_ref, scsis):
        log(DEBUG, "create_scsis")
        return map(lambda scsi: self.create_scsi(vm_ref, scsi), scsis)

    def create_scsi(self, vm_ref, scsi):
        log(DEBUG, "create_scsi")

        if scsi.attributes["feature-host"].value == "True":
            target_HBA_ref = None
            for pscsi_HBA_ref in server.xenapi.PSCSI_HBA.get_all():
                if int(scsi.attributes["devid"].value) == \
                   int(server.xenapi.PSCSI_HBA.get_physical_host(pscsi_HBA_ref)):
                    target_HBA_ref = pscsi_HBA_ref
                    break
            if target_HBA_ref is None:
                log(DEBUG, "create_scsi: scsi device not found")
                return None

            dscsi_record = {
                "VM":
                    vm_ref,
                "PSCSI_HBA":
                    target_HBA_ref,
                "assignment_mode":
                    "HOST"
            }

            return server.xenapi.DSCSI_HBA.create(dscsi_record)
        else:
            target_ref = None
            for pscsi_ref in server.xenapi.PSCSI.get_all():
                if scsi.attributes["p-dev"].value == \
                   server.xenapi.PSCSI.get_physical_HCTL(pscsi_ref):
                    target_ref = pscsi_ref
                    break
            if target_ref is None:
                log(DEBUG, "create_scsi: scsi device not found")
                return None

            dscsi_record = {
                "VM":
                    vm_ref,
                "PSCSI":
                    target_ref,
                "virtual_HCTL":
                    scsi.attributes["v-dev"].value
            }

            return server.xenapi.DSCSI.create(dscsi_record)

def get_child_by_name(exp, childname, default = None):
    try:
        return [child for child in sxp.children(exp)
                if child[0] == childname][0][1]
    except:
        return default

# Convert old sxp into new xml

class sxp2xml:

    def convert_sxp_to_xml(self, config, transient=False):
       
        devices = [child for child in sxp.children(config)
                   if len(child) > 0 and child[0] == "device"]
                   
        vbds_sxp = map(lambda x: x[1], [device for device in devices
                                        if device[1][0] in ("vbd", "tap", "tap2")])

        vifs_sxp = map(lambda x: x[1], [device for device in devices
                                        if device[1][0] == "vif"])

        vfbs_sxp = map(lambda x: x[1], [device for device in devices
                                        if device[1][0] == "vfb"])

        pcis_sxp = map(lambda x: x[1], [device for device in devices
                                        if device[1][0] == "pci"])

        scsis_sxp = map(lambda x: x[1], [device for device in devices
                                         if device[1][0] == "vscsi"])

        # Create XML Document
        
        impl = getDOMImplementation()

        document = impl.createDocument(None, "xm", None)

        # Lets make the VM tag..

        vm = document.createElement("vm")

        # Some string compatibility

        actions_after_shutdown \
            = get_child_by_name(config, "on_poweroff", "destroy")
        actions_after_reboot \
            = get_child_by_name(config, "on_reboot", "restart")
        actions_after_crash \
            = get_child_by_name(config, "on_crash", "restart")

        def conv_chk(val, vals):
            lval = val.replace("-", "_")
            if lval not in vals:
                raise ValueError("Invalid value: %s" % val)
            else:
                return lval

        actions_after_shutdown = conv_chk(actions_after_shutdown,\
                                          XEN_API_ON_NORMAL_EXIT)
        actions_after_reboot   = conv_chk(actions_after_reboot, \
                                          XEN_API_ON_NORMAL_EXIT)
        actions_after_crash    = conv_chk(actions_after_crash, \
                                          XEN_API_ON_CRASH_BEHAVIOUR)
        # Flesh out tag attributes            

        vm.attributes["is_a_template"] = "false"
        vm.attributes["auto_power_on"] = "false"
        vm.attributes["actions_after_shutdown"] \
            = actions_after_shutdown              
        vm.attributes["actions_after_reboot"] \
            = actions_after_reboot
        vm.attributes["actions_after_crash"] \
            = actions_after_crash
        vm.attributes["PCI_bus"] = ""

        vm.attributes["vcpus_max"] \
            = str(get_child_by_name(config, "vcpus", 1))
        vm.attributes["vcpus_at_startup"] \
            = str(get_child_by_name(config, "vcpus", 1))
        vm.attributes["s3_integrity"] \
            = str(get_child_by_name(config, "s3_integrity", 0))
        vm.attributes["superpages"] \
            = str(get_child_by_name(config, "superpages", 0))
        vm.attributes["pool_name"] \
            = str(get_child_by_name(config, "pool_name", "Pool-0"))

        sec_data = get_child_by_name(config, "security")
        if sec_data:
            try :
                vm.attributes['security_label'] = \
                                    security.set_security_label(sec_data[0][1][1],sec_data[0][2][1])
            except Exception, e:
                raise ValueError("Invalid security data format: %s" % str(sec_data))

        # Make the name tag

        vm.appendChild(self.make_name_tag(
            get_child_by_name(config, "name"), document))

        # Make version tag

        version = document.createElement("version")
        version.appendChild(document.createTextNode("0"))
        vm.appendChild(version)
        
        # Make pv or hvm tag

        image = get_child_by_name(config, "image")

        if image[0] == "linux":
            pv = document.createElement("pv")
            pv.attributes["kernel"] \
                = get_child_by_name(image, "kernel", "")
            pv.attributes["bootloader"] \
                = get_child_by_name(config, "bootloader", "")
            pv.attributes["ramdisk"] \
                = get_child_by_name(image, "ramdisk", "")
            pv.attributes["args"] \
                = "root=" + get_child_by_name(image, "root", "") \
                + " " + get_child_by_name(image, "args", "")
            pv.attributes["bootloader_args"] \
                = get_child_by_name(config, "bootloader_args","")

            vm.appendChild(pv)
        elif image[0] == "hvm":
            hvm = document.createElement("hvm")
            hvm.attributes["boot_policy"] = "BIOS order"

            boot_order = document.createElement("boot_param")
            boot_order.attributes["key"] = "order"
            boot_order.attributes["value"] \
                = get_child_by_name(image, "boot", "abcd")
            hvm.appendChild

            vm.appendChild(hvm)

        # Make memory tag

        memory = document.createElement("memory")

        memory_str = str(int(
            get_child_by_name(config, "memory"))*1024*1024)

        memory.attributes["static_min"] = str(0)
        memory.attributes["static_max"] = memory_str
        memory.attributes["dynamic_min"] = memory_str
        memory.attributes["dynamic_max"] = memory_str

        if get_child_by_name(config, "maxmem"):
            memory.attributes["static_max"] = \
               str(int(get_child_by_name(config, "maxmem")*1024*1024))

        vm.appendChild(memory)

        # And now the vbds

        vbds = map(lambda vbd: self.extract_vbd(vbd, document), vbds_sxp)

        map(vm.appendChild, vbds)

        # And now the vifs

        vifs = map(lambda vif: self.extract_vif(vif, document), vifs_sxp)

        map(vm.appendChild, vifs)

        # And now the pcis

        pcis = self.extract_pcis(pcis_sxp, document)

        map(vm.appendChild, pcis)

        # And now the scsis

        scsis = self.extract_scsis(scsis_sxp, document)

        map(vm.appendChild, scsis)

        # Last but not least the consoles...

        consoles = self.extract_consoles(image, document)

        map(vm.appendChild, consoles)

        vfbs = map(lambda vfb: self.extract_vfb(vfb, document), vfbs_sxp)

        map(vm.appendChild, vfbs)

        # Platform variables...

        platform = self.extract_platform(image, document)

        map(vm.appendChild, platform)

        # And now the vcpu_params

        vcpu_params = self.extract_vcpu_params(config, document)

        map(vm.appendChild, vcpu_params)

        # transient?

        if transient:
            other_config = document.createElement("other_config")
            other_config.attributes["key"] = "transient"
            other_config.attributes["value"] = "True"
            vm.appendChild(other_config)
        
        # Add it to doc_root

        document.documentElement.appendChild(vm)
        
        # We want to pull out vdis

        vdis = map(lambda vdb: self.extract_vdi(vdb, document), vbds_sxp)

        map(document.documentElement.appendChild, vdis)

        return document

    def make_name_tag(self, label_text, document):
        name = document.createElement("name")

        label = document.createElement("label")
        label.appendChild(document.createTextNode(str(label_text)))
        name.appendChild(label)

        description = document.createElement("description")
        description.appendChild(document.createTextNode(" "))
        name.appendChild(description)

        return name

    def extract_vbd(self, vbd_sxp, document):
        src = get_child_by_name(vbd_sxp, "uname")
        mode = get_child_by_name(vbd_sxp, "mode")
        name = str(src.__hash__())

        vbd = document.createElement("vbd")

        vbd.attributes["name"] = "vdb" + name
        vbd.attributes["vdi"] = "vdi" + name
        vbd.attributes["mode"] \
            = re.search("^w!{0,1}$", mode) and "RW" or "RO"
        vbd.attributes["device"] \
            = re.sub(":cdrom$", "", get_child_by_name(vbd_sxp, "dev"))
        vbd.attributes["bootable"] = "1"
        vbd.attributes["type"] \
            = re.search(":cdrom$", get_child_by_name(vbd_sxp, "dev")) \
              and "CD" or "disk"
        vbd.attributes["qos_algorithm_type"] = ""

        return vbd

    def extract_vdi(self, vbd_sxp, document):
        src = get_child_by_name(vbd_sxp, "uname")
        mode = get_child_by_name(vbd_sxp, "mode")
        name = "vdi" + str(src.__hash__())

        vdi = document.createElement("vdi")

        vdi.attributes["src"] = src
        vdi.attributes["read_only"] \
            = re.search("^w!{0,1}$", mode) and "False" or "True"
        vdi.attributes["size"] = '-1'
        vdi.attributes["type"] = "system"
        vdi.attributes["sharable"] \
            = re.search("^w!$", mode) and "True" or "False"
        vdi.attributes["name"] = name

        vdi.appendChild(self.make_name_tag(name, document))

        return vdi

    def extract_vif(self, vif_sxp, document):

        vif = document.createElement("vif")

        dev = get_child_by_name(vif_sxp, "vifname", None)

        if dev is None:
            dev = self.getFreshEthDevice()

        vif.attributes["name"] \
            = "vif" + str(dev.__hash__())
        vif.attributes["mac"] \
            = get_child_by_name(vif_sxp, "mac", "")               
        vif.attributes["mtu"] \
            = get_child_by_name(vif_sxp, "mtu", "")  
        vif.attributes["device"] = dev
        vif.attributes["qos_algorithm_type"] = ""

        policy = get_child_by_name(vif_sxp, "policy")
        label = get_child_by_name(vif_sxp, "label")

        vif.attributes["security_label"] = security.set_security_label(policy, label)

        if get_child_by_name(vif_sxp, "bridge") is not None:
            vif.attributes["network"] \
                = get_child_by_name(vif_sxp, "bridge")
        
        return vif

    def extract_vfb(self, vfb_sxp, document):

        vfb = document.createElement("console")
        vfb.attributes["protocol"] = "rfb"

        if get_child_by_name(vfb_sxp, "type", "") == "vnc":
            vfb.appendChild(self.mk_other_config(
                "type", "vnc",
                document))
            vfb.appendChild(self.mk_other_config(
                "vncunused", get_child_by_name(vfb_sxp, "vncunused", "1"),
                document))
            vfb.appendChild(self.mk_other_config(
                "vnclisten",
                get_child_by_name(vfb_sxp, "vnclisten", "127.0.0.1"),
                document))
            vfb.appendChild(self.mk_other_config(
                "vncdisplay", get_child_by_name(vfb_sxp, "vncdisplay", "0"),
                document))
            vfb.appendChild(self.mk_other_config(
                "vncpasswd", get_child_by_name(vfb_sxp, "vncpasswd", ""),
                document))

        if get_child_by_name(vfb_sxp, "type", "") == "sdl":
            vfb.appendChild(self.mk_other_config(
                "type", "sdl",
                document))
            vfb.appendChild(self.mk_other_config(
                "display", get_child_by_name(vfb_sxp, "display", ""),
                document))
            vfb.appendChild(self.mk_other_config(
                "xauthority",
                get_child_by_name(vfb_sxp, "xauthority", ""),
                document))
            vfb.appendChild(self.mk_other_config(
                "opengl", get_child_by_name(vfb_sxp, "opengl", "1"),
                document))

        return vfb

    def extract_pcis(self, pcis_sxp, document):

        pcis = []

        for pci_sxp in pcis_sxp:
            for dev_sxp in sxp.children(pci_sxp, "dev"):
                pci = document.createElement("pci")

                pci.attributes["domain"] \
                    = get_child_by_name(dev_sxp, "domain", "0")
                pci.attributes["bus"] \
                    = get_child_by_name(dev_sxp, "bus", "0")
                pci.attributes["slot"] \
                    = get_child_by_name(dev_sxp, "slot", "0")
                pci.attributes["func"] \
                    = get_child_by_name(dev_sxp, "func", "0")
                pci.attributes["vdevfn"] \
                    = get_child_by_name(dev_sxp, "vdevfn", "0")
                pci.attributes["key"] \
                    = get_child_by_name(dev_sxp, "key", "0")
                for opt in pci_opts_list_from_sxp(dev_sxp):
                    pci_opt = document.createElement("pci_opt")
                    pci_opt.attributes["key"] = opt[0]
                    pci_opt.attributes["value"] = opt[1]
                    pci.appendChild(pci_opt)

                pcis.append(pci)

        return pcis

    def extract_scsis(self, scsis_sxp, document):

        scsis = []

        for scsi_sxp in scsis_sxp:
            feature_host = sxp.child_value(scsi_sxp, "feature-host")
            for dev_sxp in sxp.children(scsi_sxp, "dev"):
                scsi = document.createElement("vscsi")

                scsi.attributes["feature-host"] \
                    = feature_host and "True" or "False"
                if feature_host:
                    scsi.attributes["devid"] \
                        = str(get_child_by_name(dev_sxp, "devid"))
                    scsis.append(scsi)
                    break
                else:
                    scsi.attributes["p-dev"] \
                        = get_child_by_name(dev_sxp, "p-dev")
                    scsi.attributes["v-dev"] \
                        = get_child_by_name(dev_sxp, "v-dev")
                    scsis.append(scsi)

        return scsis

    def mk_other_config(self, key, value, document):
        other_config = document.createElement("other_config")
        other_config.attributes["key"] = key
        other_config.attributes["value"] = value
        return other_config

    def extract_consoles(self, image, document):
        consoles = []

        if int(get_child_by_name(image, "nographic", "1")) == 1:
            return consoles
        
        if int(get_child_by_name(image, "vnc", "0")) == 1:
            console = document.createElement("console")
            console.attributes["protocol"] = "rfb"
            console.appendChild(self.mk_other_config(
                "type", "vnc",
                document))
            console.appendChild(self.mk_other_config(
                "vncunused", str(get_child_by_name(image, "vncunused", "1")),
                document))
            console.appendChild(self.mk_other_config(
                "vnclisten",
                get_child_by_name(image, "vnclisten", "127.0.0.1"),
                document))
            console.appendChild(self.mk_other_config(
                "vncdisplay", str(get_child_by_name(image, "vncdisplay", "0")),
                document))
            console.appendChild(self.mk_other_config(
                "vncpasswd", get_child_by_name(image, "vncpasswd", ""),
                document))
            consoles.append(console)          
        if int(get_child_by_name(image, "sdl", "0")) == 1:
            console = document.createElement("console")
            console.attributes["protocol"] = "rfb"
            console.appendChild(self.mk_other_config(
                "type", "sdl",
                document))
            console.appendChild(self.mk_other_config(
                "display", get_child_by_name(image, "display", ""),
                document))
            console.appendChild(self.mk_other_config(
                "xauthority", get_child_by_name(image, "xauthority", ""),
                document))
            console.appendChild(self.mk_other_config(
                "opengl", str(get_child_by_name(image, "opengl", "1")),
                document))
            consoles.append(console)
            
        return consoles


    def extract_platform(self, image, document):

        platform_keys = [
            'acpi',
            'apic',
            'boot',
            'device_model',
            'loader',
            'fda',
            'fdb',
            'keymap',
            'isa',
            'localtime',
            'monitor',
            'pae',
            'rtc_timeoffset',
            'serial',
            'soundhw',
            'stdvga',
            'usb',
            'usbdevice',
            'hpet',
            'timer_mode',
            'vpt_align',
            'viridian',
            'vhpt',
            'guest_os_type',
            'hap',
            'oos',
            'pci_msitranslate',
            'pci_power_mgmt',
            'xen_platform_pci',
            'tsc_mode'
            'description',
            'nomigrate'
        ]

        platform_configs = []
        for key in platform_keys:
            value = get_child_by_name(image, key, None)
            if value is not None:
                platform = document.createElement("platform")
                platform.attributes["key"] = key
                platform.attributes["value"] = str(value)
                platform_configs.append(platform)
 
        return platform_configs

    def extract_vcpu_params(self, config, document):
        vcpu_params = []

        vcpu_param = document.createElement("vcpu_param")
        vcpu_param.attributes["key"] = "weight"
        vcpu_param.attributes["value"] \
            = str(get_child_by_name(config, "cpu_weight", 256))
        vcpu_params.append(vcpu_param)

        vcpu_param = document.createElement("vcpu_param")
        vcpu_param.attributes["key"] = "cap"
        vcpu_param.attributes["value"] \
            = str(get_child_by_name(config, "cpu_cap", 0))
        vcpu_params.append(vcpu_param)

        cpus = get_child_by_name(config, "cpus", [])
        if type(cpus) == list:
            vcpu = 0
            for cpu in cpus:
                if cpu:
                    vcpu_param = document.createElement("vcpu_param")
                    vcpu_param.attributes["key"] = "cpumap%i" % vcpu
                    vcpu_param.attributes["value"] = str(cpu)
                    vcpu_params.append(vcpu_param)
                vcpu = vcpu + 1
        else:
            for vcpu in range(0, int(get_child_by_name(config, "vcpus", 1))):
                vcpu_param = document.createElement("vcpu_param")
                vcpu_param.attributes["key"] = "cpumap%i" % vcpu
                vcpu_param.attributes["value"] = str(cpus)
                vcpu_params.append(vcpu_param)

        return vcpu_params
    
    _eths = -1

    def getFreshEthDevice(self):
        self._eths += 1
        return "eth%i" % self._eths



#!/usr/bin/python
#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2007 Tom Wilkie <tom.wilkie@gmail.com>
#============================================================================
"""
Base class for all XenAPI classes
"""

from xen.xend.XendError import *
from xen.xend import XendAPIStore

class XendBase:
    #
    # These functions describe the object, and what is exposed via the API
    #
    def getClass(self):
        return "Base"
    
    def getAttrRO(self):
        return ['uuid']

    def getAttrRW(self):
        return []

    def getAttrInst(self):
        return []

    def getMethods(self):
        return ["get_record"]

    def getFuncs(self):
        return ["get_all", "get_by_uuid", "get_all_records"]

    getClass    = classmethod(getClass)
    getAttrRO   = classmethod(getAttrRO)
    getAttrRW   = classmethod(getAttrRW)
    getAttrInst = classmethod(getAttrInst)
    getMethods  = classmethod(getMethods)
    getFuncs    = classmethod(getFuncs)
    
    def __init__(self, uuid, record):
        self.__uuid = uuid
        
        # First check this class implements all the correct methods:
        for attr_ro in self.getAttrRO() + self.getAttrRW():
            if not hasattr(self, "get_%s" % attr_ro):
                raise ImplementationError(self.getClass(),
                                          "get_%s" % attr_ro)

        for attr_rw in self.getAttrRW():
            if not hasattr(self, "set_%s" % attr_rw):
                raise ImplementationError(self.getClass(),
                                          "set_%s" % attr_rw)

        for method in self.getMethods():
            if not hasattr(self, method):
                raise ImplementationError(self.getClass(),
                                          method)

        for func in self.getFuncs():
            if not hasattr(self.__class__, func):
                raise ImplementationError(self.getClass(),
                                          func)

        # Next check that the class is being created with the correct
        # parameters
        if not isinstance(record, dict):
            raise CreateUnspecifiedAttributeError(
                    "record" , self.getClass())
        
        for attr_inst in self.getAttrInst():
            if attr_inst not in record:
                raise CreateUnspecifiedAttributeError(
                    attr_inst, self.getClass())
            setattr(self, attr_inst, record[attr_inst])

        # Finally register it
        XendAPIStore.register(uuid, self.getClass(), self)

    def destroy(self):
        XendAPIStore.deregister(self.get_uuid(), self.getClass())

    def get_uuid(self):
        return self.__uuid

    def get_record(self):
        keys = self.getAttrRO() + self.getAttrRW()
        return dict([(key, getattr(self, "get_%s" % key)())
                     for key in keys])

    #
    # Class methods
    #

    def get_all(cls):
        return XendAPIStore.get_all_uuid(cls.getClass())

    def get_by_uuid(cls, uuid):
        # Sanity check the uuid is one of us
        me = XendAPIStore.get(uuid, cls.getClass())
        if me is not None and me.getClass() == cls.getClass():
            # In OSS, ref == uuid
            return uuid
        else:
            raise ValueError("Big Error.. TODO!")

    def get_all_records(cls):
        return dict([(inst.get_uuid(), inst.get_record())
                     for inst in XendAPIStore.get_all(cls.getClass())])

    get_all = classmethod(get_all)
    get_by_uuid = classmethod(get_by_uuid)
    get_all_records = classmethod(get_all_records)



#
# Copyright (c) 2001, 2002, 2003, 2004 Python Software Foundation; All Rights Reserved
#
# PSF LICENSE AGREEMENT FOR PYTHON 2.3
# ------------------------------------
# 
# 1. This LICENSE AGREEMENT is between the Python Software Foundation
# ("PSF"), and the Individual or Organization ("Licensee") accessing and
# otherwise using Python 2.3 software in source or binary form and its
# associated documentation.
# 
# 2. Subject to the terms and conditions of this License Agreement, PSF
# hereby grants Licensee a nonexclusive, royalty-free, world-wide
# license to reproduce, analyze, test, perform and/or display publicly,
# prepare derivative works, distribute, and otherwise use Python 2.3
# alone or in any derivative version, provided, however, that PSF's
# License Agreement and PSF's notice of copyright, i.e., "Copyright (c)
# 2001, 2002, 2003, 2004 Python Software Foundation; All Rights Reserved" are
# retained in Python 2.3 alone or in any derivative version prepared by
# Licensee.
# 
# 3. In the event Licensee prepares a derivative work that is based on
# or incorporates Python 2.3 or any part thereof, and wants to make
# the derivative work available to others as provided herein, then
# Licensee hereby agrees to include in any such work a brief summary of
# the changes made to Python 2.3.
# 
# 4. PSF is making Python 2.3 available to Licensee on an "AS IS"
# basis.  PSF MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, PSF MAKES NO AND
# DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
# FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF PYTHON 2.3 WILL NOT
# INFRINGE ANY THIRD PARTY RIGHTS.
# 
# 5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON
# 2.3 FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
# A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON 2.3,
# OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.
# 
# 6. This License Agreement will automatically terminate upon a material
# breach of its terms and conditions.
# 
# 7. Nothing in this License Agreement shall be deemed to create any
# relationship of agency, partnership, or joint venture between PSF and
# Licensee.  This License Agreement does not grant permission to use PSF
# trademarks or trade name in a trademark sense to endorse or promote
# products or services of Licensee, or any third party.
# 
# 8. By copying, installing or otherwise using Python 2.3, Licensee
# agrees to be bound by the terms and conditions of this License
# Agreement.
# 
# Modifications: Copyright (c) 2005 Christian Limpach <Christian.Limpach@cl.cam.ac.uk>
# - add support for excluding a list of file descriptors from being
#   closed, allowing access to those file descriptors from the command.
#

"""Spawn a command with pipes to its stdin, stdout, and optionally stderr.

The normal os.popen(cmd, mode) call spawns a shell command and provides a
file interface to just the input or output of the process depending on
whether mode is 'r' or 'w'.  This module provides the functions xpopen2(cmd)
and xpopen3(cmd) which return two or three pipes to the spawned command.
Optionally exclude a list of file descriptors from being closed, allowing
access to those file descriptors from the command.
"""

import os
import sys

try:
    MAXFD = os.sysconf('SC_OPEN_MAX')
except (AttributeError, ValueError):
    MAXFD = 256

_active = []

def _cleanup():
    for inst in _active[:]:
        inst.poll()

class xPopen3:
    """Class representing a child process.  Normally instances are created
    by the factory functions popen2() and popen3()."""

    sts = -1                    # Child not completed yet

    def __init__(self, cmd, capturestderr=False, bufsize=-1, passfd=(), env=None):
        """The parameter 'cmd' is the shell command to execute in a
        sub-process.  The 'capturestderr' flag, if true, specifies that
        the object should capture standard error output of the child process.
        The default is false.  If the 'bufsize' parameter is specified, it
        specifies the size of the I/O buffers to/from the child process."""
        _cleanup()
        self.passfd = passfd
        p2cread, p2cwrite = os.pipe()
        c2pread, c2pwrite = os.pipe()
        if capturestderr:
            errout, errin = os.pipe()
        self.pid = os.fork()
        if self.pid == 0:
            # Child
            os.dup2(p2cread, 0)
            os.dup2(c2pwrite, 1)
            if capturestderr:
                os.dup2(errin, 2)
            self._run_child(cmd, env)
        os.close(p2cread)
        self.tochild = os.fdopen(p2cwrite, 'w', bufsize)
        os.close(c2pwrite)
        self.fromchild = os.fdopen(c2pread, 'r', bufsize)
        if capturestderr:
            os.close(errin)
            self.childerr = os.fdopen(errout, 'r', bufsize)
        else:
            self.childerr = None
        _active.append(self)

    def _run_child(self, cmd, env):
        if isinstance(cmd, basestring):
            cmd = ['/bin/sh', '-c', cmd]
        for i in range(3, MAXFD):
            if i in self.passfd:
                continue
            try:
                os.close(i)
            except OSError:
                pass
        try:
            if env is None:
                os.execvp(cmd[0], cmd)
            else:
                os.execvpe(cmd[0], cmd, env)
        finally:
            os._exit(127)

    def poll(self):
        """Return the exit status of the child process if it has finished,
        or -1 if it hasn't finished yet."""
        if self.sts < 0:
            try:
                pid, sts = os.waitpid(self.pid, os.WNOHANG)
                if pid == self.pid:
                    self.sts = sts
                    _active.remove(self)
            except os.error:
                pass
        return self.sts

    def wait(self):
        """Wait for and return the exit status of the child process."""
        if self.sts < 0:
            pid, sts = os.waitpid(self.pid, 0)
            if pid == self.pid:
                self.sts = sts
                _active.remove(self)
        return self.sts


def xpopen2(cmd, bufsize=-1, mode='t', passfd=[], env=None):
    """Execute the shell command 'cmd' in a sub-process.  If 'bufsize' is
    specified, it sets the buffer size for the I/O pipes.  The file objects
    (child_stdout, child_stdin) are returned."""
    inst = xPopen3(cmd, False, bufsize, passfd, env)
    return inst.fromchild, inst.tochild

def xpopen3(cmd, bufsize=-1, mode='t', passfd=[], env=None):
    """Execute the shell command 'cmd' in a sub-process.  If 'bufsize' is
    specified, it sets the buffer size for the I/O pipes.  The file objects
    (child_stdout, child_stdin, child_stderr) are returned."""
    inst = xPopen3(cmd, True, bufsize, passfd, env)
    return inst.fromchild, inst.tochild, inst.childerr

def call(*popenargs, **kwargs):
    """Run command with arguments.  Wait for command to complete, then
    return the status.

    The arguments are the same as for the xPopen3 constructor.  Example:

    status = call("ls -l")
    """
    return xPopen3(*popenargs, **kwargs).wait()



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2006,2007 International Business Machines Corp.
# Author: Stefan Berger <stefanb@us.ibm.com>
#============================================================================

import threading
import xsconstants

class XSPolicy:
    """
       The base policy class for all policies administered through
       XSPolicyAdmin.
    """

    def __init__(self, name=None, ref=None):
        self.lock = threading.Lock()
        self.ref = ref
        self.name = name
        if ref:
            from xen.xend.XendXSPolicy import XendXSPolicy
            self.xendxspolicy = XendXSPolicy(self, {}, ref)
        else:
            self.xendxspolicy = None

    def grab_lock(self):
        self.lock.acquire()

    def unlock(self):
        self.lock.release()

    def get_ref(self):
        return self.ref

    def destroy(self):
        if self.xendxspolicy:
            self.xendxspolicy.destroy()

    # All methods below should be overwritten by the inheriting class

    def isloaded(self):
        return False

    def loadintohv(self):
        return xsconstants.XSERR_POLICY_LOAD_FAILED

    def get_type(self):
        return xsconstants.XS_POLICY_NONE

    def get_type_name(self):
        return ""

    def update(self, repr_new):
        return -xsconstants.XSERR_GENERAL_FAILURE, ""



#!/usr/bin/python
#
# Sched-credit tests modified from SEDF tests
#

import re

from XmTestLib import *

paramsRE = re.compile(r'^[^ ]* *[^ ]* *([^ ]*) *([^ ]*)$')

def get_sched_credit_params(domain):
    status, output = traceCommand("xm sched-credit -d %s | tail -1" %
                                  domain.getName())

    if status != 0:
        FAIL("Getting sched-credit parameters return non-zero rv (%d)",
             status)

    m = paramsRE.match(output)
    if not m:
        FAIL("xm sched-credit gave bad output")
    weight = int(m.group(1))
    cap = int(m.group(2))
    return (weight, cap)

def set_sched_credit_weight(domain, weight):
    status, output = traceCommand("xm sched-credit -d %s -w %d" %(domain.getName(), weight))
    return status

def set_sched_credit_cap(domain, cap):
    status, output = traceCommand("xm sched-credit -d %s -c %d" %(domain.getName(), cap))
    return status


domain = XmTestDomain()

try:
    domain.start(noConsole=True)
except DomainError, e:
    if verbose:
        print "Failed to create test domain because:"
        print e.extra
    FAIL(str(e))

# check default param values
(weight, cap) = get_sched_credit_params(domain)

if weight != 256:
    FAIL("default weight is 256 (got %d)", weight)
if cap != 0:
    FAIL("default cap is 0 (got %d)", cap)

# set new parameters
status = set_sched_credit_weight(domain, 512)
if status != 0:
    FAIL("Setting sched-credit weight return non-zero rv (%d)", status)

status = set_sched_credit_cap(domain, 100)
if status != 0:
    FAIL("Setting sched-credit cap return non-zero rv (%d)", status)

# check new param values
(weight, cap) = get_sched_credit_params(domain)

if weight != 512:
    FAIL("expected weight of 512 (got %d)", weight)
if cap != 100:
    FAIL("expected cap of 100 (got %d)", cap)

# Stop the domain (nice shutdown)
domain.stop()



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2004, 2005 Mike Wray <mike.wray@hp.com>
#============================================================================

import types
import StringIO

from xen.xend import sxp

class ArgError(StandardError):
    pass

class Args:
    """Argument encoding support for HTTP.
    """
    
    def __init__(self, paramspec, keyspec):
        self.arg_ord = []
        self.arg_dict = {}
        self.key_ord = []
        self.key_dict = {}
        for (name, typ) in paramspec:
                self.arg_ord.append(name)
                self.arg_dict[name] = typ
        for (name, typ) in keyspec:
                self.key_ord.append(name)
                self.key_dict[name] = typ

    def get_args(self, d, xargs=None):
        args = {}
        keys = {}
        params = []
        if xargs:
            self.split_args(xargs, args, keys)
        self.split_args(d, args, keys)
        for a in self.arg_ord:
            if a in args:
                params.append(args[a])
            else:
                raise ArgError('Missing parameter: %s' % a)
        return (params, keys)

    def split_args(self, d, args, keys):
        for (k, v) in d.items():
            if k in self.arg_dict:
                typ = self.arg_dict[k]
                val = self.coerce(typ, v)
                args[k] = val
            elif k in self.key_dict:
                typ = self.key_dict[k]
                val = self.coerce(typ, v)
                keys[k] = val
            else:
                raise ArgError('Invalid parameter: %s' % k)

    def get_form_args(self, f, xargs=None):
        d = {}
        for (k, v) in f.items():
            if ((k not in self.arg_dict) and
                (k not in self.key_dict)):
                continue
            if isinstance(v, types.ListType):
                n = len(v)
                if n == 0:
                    continue
                elif n == 1:
                    val = v[0]
                else:
                    raise ArgError('Too many values for %s' % k)
            else:
                val = v
            d[k] = val
        return self.get_args(d, xargs=xargs)

    def coerce(self, typ, v):
        try:
            if typ == 'int':
                val = int(v)
            elif typ == 'long':
                val = long(v)
            elif typ == 'str':
                val = str(v)
            elif typ == 'sxpr':
                val = self.sxpr(v)
            elif typ == 'bool':
                val = self.bool(v)
            else:
                raise ArgError('invalid type:' + str(typ))
            return val
        except ArgError:
            raise
        except StandardError, ex:
            raise ArgError(str(ex))

    def bool(self, v):
        return (v.lower() in ['on', 'yes', '1', 'true'])

    def sxpr(self, v):
        if isinstance(v, types.ListType):
            val = v
        elif isinstance(v, types.FileType) or hasattr(v, 'readline'):
            val = self.sxpr_file(v)
        elif isinstance(v, types.StringType):
            val = self.sxpr_file(StringIO.StringIO(v))
        else:
            val = str(v)
        return val

    def sxpr_file(self, fin):
        try:
            vals = sxp.parse(fin)
        except:
            raise ArgError('Coercion to sxpr failed')
        if len(vals) == 1:
            return vals[0]
        else:
            raise ArgError('Too many sxprs')

    def call_with_args(self, fn, args, xargs=None):
        (params, keys) = self.get_args(args, xargs=xargs)
        return fn(*params, **keys)

    def call_with_form_args(self, fn, fargs, xargs=None):
        (params, keys) = self.get_form_args(fargs, xargs=xargs)
        return fn(*params, **keys)

class ArgFn(Args):
    """Represent a remote HTTP operation as a function.
    Used on the client.
    """

    def __init__(self, fn, paramspec, keyspec = None):
        if keyspec == None:
            keyspec = {}
        Args.__init__(self, paramspec, keyspec)
        self.fn = fn

    def __call__(self, fargs, xargs=None):
        return self.call_with_args(self.fn, fargs, xargs=xargs)
    
class FormFn(Args):
    """Represent an operation as a function over a form.
    Used in the HTTP server.
    """

    def __init__(self, fn, paramspec, keyspec = None):
        if keyspec == None:
            keyspec = {}
        Args.__init__(self, paramspec, keyspec)
        self.fn = fn

    def __call__(self, fargs, xargs=None):
        return self.call_with_form_args(self.fn, fargs, xargs=xargs)



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2006 International Business Machines Corp.
# Author: Bryan D. Payne <bdpayne@us.ibm.com>
#============================================================================

"""Tests the security settings for a domain and its resources.
"""
import sys
import xen.util.xsm.xsm as security
from xen.xm import create
from xen.xend import sxp
from xen.util import xsconstants
from xen.xm.opts import OptionError

def help():
    return """
    This program checks each resource listed in the configfile
    to see if the domain created by the configfile can access
    the resources.  The status of each resource is listed
    individually along with the final security decision."""


def check_domain_label(config, verbose):
    """All that we need to check here is that the domain label exists and
       is not null when security is on.  Other error conditions are
       handled when the config file is parsed.
    """
    answer = 0
    default_label = None
    secon = 0
    if security.on() == xsconstants.XS_POLICY_ACM:
        default_label = security.ssidref2label(security.NULL_SSIDREF)
        secon = 1

    # get the domain acm_label
    dom_label = None
    dom_name = None
    for x in sxp.children(config):
        if sxp.name(x) == 'security':
            dom_label = sxp.child_value(sxp.name(sxp.child0(x)), 'label')
        if sxp.name(x) == 'name':
            dom_name = sxp.child0(x)

    # sanity check on domain label
    if verbose:
        print "Checking domain:"
    if (not secon) and (not dom_label):
        answer = 1
        if verbose:
            print "   %s: PERMITTED" % (dom_name)
    elif (secon) and (dom_label) and (dom_label != default_label):
        answer = 1
        if verbose:
            print "   %s: PERMITTED" % (dom_name)
    else:
        print "   %s: DENIED" % (dom_name)
        if not secon:
            print "   --> Security off, but domain labeled"
        else:
            print "   --> Domain not labeled"
        answer = 0

    return answer

def config_security_check(config, verbose):
    """Checks each resource listed in the config to see if the active
       policy will permit creation of a new domain using the config.
       Returns 1 if the config passes all tests, otherwise 0.
    """
    answer = 1

    # get the domain acm_label
    domain_label = None
    domain_policy = None
    for x in sxp.children(config):
        if sxp.name(x) == 'security':
            domain_label = sxp.child_value(sxp.name(sxp.child0(x)), 'label')
            domain_policy = sxp.child_value(sxp.name(sxp.child0(x)), 'policy')

    # if no domain label, use default
    if not domain_label and security.on() == xsconstants.XS_POLICY_ACM:
        try:
            domain_label = security.ssidref2label(security.NULL_SSIDREF)
        except:
            import traceback
            traceback.print_exc(limit=1)
            return 0
        domain_policy = 'NULL'
    elif not domain_label:
        domain_label = ""
        domain_policy = 'NULL'

    if verbose:
        print "Checking resources:"

    # build a list of all resources in the config file
    resources = []
    for x in sxp.children(config):
        if sxp.name(x) == 'device':
            if sxp.name(sxp.child0(x)) == 'vbd':
                resources.append(sxp.child_value(sxp.child0(x), 'uname'))

    # perform a security check on each resource
    for resource in resources:
        try:
            security.res_security_check(resource, domain_label)
            if verbose:
                print "   %s: PERMITTED" % (resource)

        except security.XSMError:
            print "   %s: DENIED" % (resource)
            (poltype, res_label, res_policy) = security.get_res_label(resource)
            if not res_label:
                res_label = ""
            print "   --> res: %s (%s:%s)" % (str(res_label),
                                           str(poltype), str(res_policy))
            print "   --> dom: %s (%s:%s)" % (str(domain_label),
                                           str(poltype), str(domain_policy))

            answer = 0

    return answer


def main (argv):
    if len(argv) != 2:
        raise OptionError('Invalid number of arguments')
    
    passed = 0
    (opts, config) = create.parseCommandLine(argv)
    if check_domain_label(config, verbose=1):
        if config_security_check(config, verbose=1):
            passed = 1
    else:
        print "Checking resources: (skipped)"
        
    if passed:
        print "Dry Run: PASSED"
    else:
        print "Dry Run: FAILED"
        sys.exit(-1)

if __name__ == '__main__':
    try:
        main(sys.argv)
    except Exception, e:
        sys.stderr.write('Error: %s\n' % str(e))
        sys.exit(-1)



import sys
from xen.util import xsconstants
from xen.xend.XendLogging import log

class XSMError(Exception):
    def __init__(self,value):
        self.value = value
    def __str__(self):
        return repr(self.value)


security_dir_prefix = "";
policy_dir_prefix = "";
active_policy = "";
NULL_SSIDREF = 0;

#Functions exported through XML-RPC
xmlrpc_exports = [
  'set_resource_label',
  'get_resource_label',
  'list_labels',
  'get_labeled_resources',
  'set_policy',
  'reset_policy',
  'get_policy',
  'activate_policy',
  'rm_bootpolicy',
  'get_xstype',
  'get_domain_label',
  'set_domain_label'
]

def err(msg):
    """Raise XSM-dummy exception.
    """
    raise XSMError(msg)

def on():
    return 0

def ssidref2label(ssidref):
    return 0

def label2ssidref(label, policy, type):
    return 0

def res_security_check(resource, domain_label):
    return 1

def get_res_security_details(resource):
    return ("","","")

def get_res_label(resource):
    return ("","")

def res_security_check_xapi(rlabel, rssidref, rpolicy, xapi_dom_label):
    return 1

def parse_security_label(security_label):
    return ""

def calc_dom_ssidref_from_info(info):
    return ""

def set_security_label(policy, label):
    return ""

def ssidref2security_label(ssidref):
    return ""

def has_authorization(ssidref):
    return True

def get_security_label(self, xspol=None):
    return ""

def get_resource_label_xapi(resource):
    return ""

def get_labeled_resources_xapi():
    return {}

def set_resource_label_xapi(resource, reslabel_xapi, oldlabel_xapi):
    err("Command not supported under XSM 'dummy' module.")

def format_resource_label(res):
    return ""

def set_resource_label(resource, policytype, policyref, reslabel,
                       oreslabel = None):
    err("Command not supported under XSM 'dummy' module.")

def get_resource_label(resource):
    return ""

def list_labels(policy_name, ltype):
    return []

def get_labeled_resources():
    return {}

def set_policy(xs_type, xml, flags, overwrite):
    err("Command not supported under xsm 'dummy' module.")

def reset_policy():
    err("Command not supported under xsm 'dummy' module.")

def get_policy():
    return "", 0

def activate_policy():
    err("Command not supported under xsm 'dummy' module.")

def rm_bootpolicy():
    err("Command not supported under xsm 'dummy' module.")

def get_xstype():
    return 0

def get_domain_label(domain):
    return ""

def set_domain_label():
    err("Command not supported under xsm 'dummy' module.")

def dump_policy():
    pass

def dump_policy_file():
    pass

def get_ssid(domain):
    err("No ssid has been assigned to any domain under xsm dummy module.")

def security_label_to_details(res_label):
    return ("","","")



#!/usr/bin/env python
#
# Helper functions for dealing with the sxp representation of devices

import types

def dev_dict_to_sxp(dev):
    def f((key, val)):
        if isinstance(val, types.ListType):
            return map(lambda x: [key, x], val)
        return [[key, val]]
    dev_sxp = ['dev'] + reduce(lambda x, y: x + y, map(f, dev.items()))
    return dev_sxp



#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Woody Marvel <marvel@us.ibm.com>
#	  Li Ge <lge@us.ibm.com>

# Positive Test:
# Test Description:
# 1. Create a domain
# 2. Destroy the domain by id
 
import sys
import re
import time

from XmTestLib import *

# Create a domain (default XmTestDomain, with our ramdisk)
domain = XmTestDomain()

# Start it
try:
    domain.start(noConsole=True)
except DomainError, e:
    if verbose:
        print "Failed to create test domain because:"
        print e.extra
    FAIL(str(e))

# destroy domain - positive test
status, output = traceCommand("xm destroy %s" % domain.getId())
if status != 0:
    FAIL("xm destroy returned invalid %i != 0" % status)



#!/usr/bin/env python
#
# Copyright 2001-2002 by Vinay Sajip. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Vinay Sajip
# not be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# VINAY SAJIP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# VINAY SAJIP BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# This file is part of the Python logging distribution. See
# http://www.red-dove.com/python_logging.html
#
"""
A test harness for the logging module. Tests BufferingHandler, BufferingFormatter.

Copyright (C) 2001-2002 Vinay Sajip. All Rights Reserved.
"""
import logging, logging.handlers

class XMLFormatter(logging.BufferingFormatter):
        """
        A formatter which formats a set of records using XML, using an example DTD called "logging.dtd".
        """
        def __init__(self):
            fmtstr = """
    <event name="%(name)s" level="%(levelno)d" filename="%(filename)s" lineno="%(lineno)d">
    <message>%(message)s</message>
    </event>"""
            logging.BufferingFormatter.__init__(self, logging.Formatter(fmtstr))

#   def formatHeader(self, records):
#       return """
#<?xml version="1.0" ?><!DOCTYPE eventSet SYSTEM "logging.dtd">
#<eventSet xmlns="http://www.red-dove.com/logging">"""
#
#   def formatFooter(self, records):
#       return "</eventSet>"

class XMLHandler(logging.handlers.BufferingHandler):
    def __init__(self, capacity):
        logging.handlers.BufferingHandler.__init__(self, capacity)
        self.setFormatter(XMLFormatter())

    def flush(self):
        if len(self.buffer) > 0:
            file = open("events.xml","w")
            file.write(self.formatter.format(self.buffer))
            file.close()
            self.buffer = []

def main():
    logger = logging.getLogger("")
    logger.setLevel(logging.DEBUG)
    xh = XMLHandler(10)
    logger.addHandler(xh)
    for i in xrange(100):
        logger.info("Info index = %d", i)
    xh.close()
    logger.removeHandler(xh)

if __name__ == "__main__":
    main()



import os
import re
import socket
import struct
import errno

##### Networking-related functions

def get_defaultroute():
    fd = os.popen('/sbin/ip route list 2>/dev/null')
    for line in fd.readlines():
        m = re.search('^default via ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+) dev ([^ ]*)',
                      line)
        if m:
            return [m.group(1), m.group(2)]
    return [None, None]

def get_current_ipaddr(dev='defaultroute'):
    """Get the primary IP address for the given network interface.

    dev     network interface (default: default route device)

    returns interface address as a string
    """
    if dev == 'defaultroute':
        dev = get_defaultroute()[1]
    if not dev:
        return
    fd = os.popen( '/sbin/ifconfig ' + dev + ' 2>/dev/null' )
    for line in fd.readlines():
        m = re.search( '^\s+inet addr:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+).*',
                       line )
        if m:
            return m.group(1)
    return None

def get_current_ipmask(dev='defaultroute'):
    """Get the primary IP netmask for a network interface.

    dev     network interface (default: default route device)

    returns interface netmask as a string
    """
    if dev == 'defaultroute':
        dev = get_defaultroute()[1]
    if not dev:
        return
    fd = os.popen( '/sbin/ifconfig ' + dev + ' 2>/dev/null' )
    for line in fd.readlines():
        m = re.search( '^.+Mask:([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+).*',
                       line )
        if m:
            return m.group(1)
    return None

def get_current_ipgw(dev='defaultroute'):
    """Get the IP gateway for a network interface.

    dev     network interface (default: default route device)

    returns gateway address as a string
    """
    if dev == 'defaultroute':
        return get_defaultroute()[0]
    if not dev:
        return
    fd = os.popen( '/sbin/route -n' )
    for line in fd.readlines():
        m = re.search( '^\S+\s+([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)' +
                       '\s+\S+\s+\S*G.*' + dev + '.*', line )
        if m:
            return m.group(1)
    return None

def inet_aton(addr):
    """Convert an IP addr in IPv4 dot notation into an int.

    addr    IP address as a string

    returns integer
    """
    b = socket.inet_aton(addr)
    return struct.unpack('!I', b)[0]

def inet_ntoa(n):
    """Convert an int into an IP addr in IPv4 dot notation.

    n       IP address

    returns string
    """
    b = struct.pack('!I', n)
    return socket.inet_ntoa(b)

def add_offset_to_ip(addr, offset):
    """Add a numerical offset to an IP addr in IPv4 dot notation.

    addr    IP address
    offset  offset to add

    returns new address
    """
    n = inet_aton(addr)
    n += offset
    return inet_ntoa(n)

def check_subnet( ip, network, netmask ):
    """Check if an IP address is in the subnet defined by
    a network address and mask'.

    ip      IP adress
    network network address
    netmask network mask
    
    returns 1 if it is in the subnet, 0 if not
    """
    n_ip = inet_aton(ip)
    n_net = inet_aton(network)
    n_mask = inet_aton(netmask)
    return (n_ip & n_mask) == (n_net & n_mask)




#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Dan Smith <danms@us.ibm.com>

from XmTestLib import *

if ENABLE_HVM_SUPPORT:
    SKIP("Block-attach not supported for HVM domains")

# Mount /dev/ram0

s, o = traceCommand("mkfs -j /dev/ram0")
if s != 0:
    FAIL("Unable to mkfs -j /dev/ram0")

s, o = traceCommand("mkdir -p mnt");
if s != 0:
    FAIL("Unable to create ./mnt")

s, o = traceCommand("mount /dev/ram0 mnt -o rw")
if s != 0:
    FAIL("Unable to mount /dev/ram0 on ./mnt")

# Now try to start a DomU with write access to /dev/ram0

config = {"disk":"phy:/dev/ram0,xvda1,w"}

domain = XmTestDomain(extraConfig=config);

try:
    domain.start()
    s, o = traceCommand("umount mnt")
    FAIL("Bug #331: Started a DomU with write access to a rw mounted block device")
except DomainError, e:
    s, o = traceCommand("umount mnt")

domain.destroy()



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (c) 2009 Fujitsu Technology Solutions.
#============================================================================

""" CPU Pool support including XEN-API and Legacy API.
"""

import types
import threading
import re
import xen.lowlevel.xc
import XendNode
import XendDomain
from xen.xend.XendLogging import log
from xen.xend.XendBase import XendBase
from xen.xend import XendAPIStore
from xen.xend.XendConstants import XS_POOLROOT
from xen.xend import uuid as genuuid
from xen.xend.XendError import VmError, XendAPIError, PoolError
from xen.xend.xenstore.xstransact import xstransact
from xen.util.sxputils import sxp2map, map2sxp


XEND_ERROR_INTERNAL             = 'INTERNAL_ERROR'
XEND_ERROR_UNKOWN_SCHED_POLICY  = 'UNKOWN_SCHED_POLICY'
XEND_ERROR_BAD_POOL_STATE       = 'POOL_BAD_STATE'
XEND_ERROR_POOL_PARAM           = 'PARAMETER_ERROR'
XEND_ERROR_INSUFFICIENT_CPUS    = 'INSUFFICIENT_CPUS'
XEND_ERROR_POOL_RECONF          = 'POOL_RECONF'
XEND_ERROR_INVALID_CPU          = 'INVAILD_CPU'
XEND_ERROR_LAST_CPU_NOT_REM     = 'LAST_CPU_NOT_REMOVEABLE'


XEN_SCHEDULER_TO_ID = {
    'credit2': xen.lowlevel.xc.XEN_SCHEDULER_CREDIT2,
    'credit' : xen.lowlevel.xc.XEN_SCHEDULER_CREDIT,
    'sedf'   : xen.lowlevel.xc.XEN_SCHEDULER_SEDF,
    }

xc = xen.lowlevel.xc.xc()

class XendCPUPool(XendBase):
    """ CPU Pool management.
        @ivar pool_lock: Lock to secure modification of pool data
        @type pool_lock: Rlock
    """

    pool_lock = threading.RLock()

    def getClass(cls):
        return "cpu_pool"

    def getAttrRO(cls):
        attrRO = ['resident_on',
                  'started_VMs',
                  'host_CPUs',
                  'activated',
                 ]
        return XendBase.getAttrRO() + attrRO

    def getAttrRW(cls):
        attrRW = ['name_label',
                  'name_description',
                  'auto_power_on',
                  'ncpu',
                  'sched_policy',
                  'proposed_CPUs',
                  'other_config',
                 ]
        return XendBase.getAttrRW() + attrRW

    def getMethods(cls):
        methods = ['destroy',
                   'activate',
                   'deactivate',
                   'add_host_CPU_live',
                   'remove_host_CPU_live',
                   'add_to_proposed_CPUs',
                   'remove_from_proposed_CPUs',
                   'add_to_other_config',
                   'remove_from_other_config',
                  ]
        return XendBase.getMethods() + methods

    def getFuncs(cls):
        funcs = ['create',
                 'get_by_name_label',
                ]
        return XendBase.getFuncs() + funcs

    getClass    = classmethod(getClass)
    getAttrRO   = classmethod(getAttrRO)
    getAttrRW   = classmethod(getAttrRW)
    getMethods  = classmethod(getMethods)
    getFuncs    = classmethod(getFuncs)


    #
    # XenAPI function calls
    #

    def create(cls, record):
        """ Create a new managed pool instance.
            @param record: attributes of pool
            @type record:  dict
            @return: uuid of created pool
            @rtype:  str
        """
        new_uuid = genuuid.createString()
        XendCPUPool(record, new_uuid)
        XendNode.instance().save_cpu_pools()
        return new_uuid

    create = classmethod(create)


    def get_by_name_label(cls, name_label):
        """ Query a Pool(ref) by its name.
            @return: ref of pool
            @rtype:  str
        """
        cls.pool_lock.acquire()
        try:
            return [ inst.get_uuid()
                     for inst in XendAPIStore.get_all(cls.getClass())
                     if inst.name_label == name_label
                   ]
        finally:
            cls.pool_lock.release()

    get_by_name_label = classmethod(get_by_name_label)


    def get_cpu_pool_by_cpu_ref(cls, host_cpu):
        """ Query cpu_pool ref the given cpu belongs to.
            @param host_cpu: ref of host_cpu to lookup
            @type host_cpu:  str
            @return: list cpu_pool refs (list contains not more than one element)
            @rtype:  list of str
        """
        node = XendNode.instance()
        cpu_nr = node.get_host_cpu_field(host_cpu, 'number')
        for pool_rec in xc.cpupool_getinfo():
            if cpu_nr in pool_rec['cpulist']:
                # pool found; return the ref
                return cls.query_pool_ref(pool_rec['cpupool'])
        return []

    get_cpu_pool_by_cpu_ref = classmethod(get_cpu_pool_by_cpu_ref)


    def get_all_managed(cls):
        """ Query all managed pools.
            @return: uuids of all managed pools
            @rtype:  list of str
        """
        cls.pool_lock.acquire()
        try:
            managed_pools = [ inst.get_uuid()
                              for inst in XendAPIStore.get_all(cls.getClass())
                              if inst.is_managed() ]
        finally:
            cls.pool_lock.release()
        return managed_pools

    get_all_managed = classmethod(get_all_managed)


    #
    # XenAPI methods calls
    #

    def __init__(self, record, new_uuid, managed_pool=True):
        XendBase.__init__(self, new_uuid, record)
        try:
            self._managed = managed_pool
            self.name_label = None

            name = record.get('name_label', 'Pool-Unnamed')
            self._checkName(name)
            self.name_label = name
            self.name_description = record.get('name_description',
                                               self.name_label)
            self.proposed_cpus = [ int(cpu)
                                   for cpu in record.get('proposed_CPUs', []) ]
            self.auto_power_on = bool(record.get('auto_power_on', False))
            self.ncpu = int(record.get('ncpu', 1))
            self.sched_policy = record.get('sched_policy', '')
            self.other_config = record.get('other_config', {})
        except Exception, ex:
            XendBase.destroy(self)
            raise ex


    def get_resident_on(self):
        """ Always return uuid of own node.
            @return: uuid of this node
            @rytpe:  str
        """
        return XendNode.instance().uuid

    def get_started_VMs(self):
        """ Query all VMs currently assigned to pool.
            @return: ref of all VMs assigned to pool; if pool is not active,
                     an empty list will be returned
            @rtype:  list of str
        """
        if self.get_activated():
            # search VMs related to this pool
            pool_id = self.query_pool_id()
            started_VMs = [ vm.get_uuid()
                            for vm in XendDomain.instance().list('all')
                            if vm.get_cpu_pool() == pool_id ]
        else:
            # pool not active, so it couldn't have any started VMs
            started_VMs = []

        return started_VMs

    def get_host_CPUs(self):
        """ Query all cpu refs of this pool currently asisgned .
            - Read pool id of this pool from xenstore
            - Read cpu configuration from hypervisor
            - lookup cpu number -> cpu ref
            @return: host_cpu refs
            @rtype:  list of str
        """
        if self.get_activated():
            node = XendNode.instance()
            pool_id = self.query_pool_id()
            if pool_id == None:
                raise PoolError(XEND_ERROR_INTERNAL,
                                [self.getClass(), 'get_host_CPUs'])
            cpus = []
            for pool_rec in xc.cpupool_getinfo():
                if pool_rec['cpupool'] == pool_id:
                    cpus = pool_rec['cpulist']

            # query host_cpu ref for any cpu of the pool
            host_CPUs = [ cpu_ref
                          for cpu_ref in node.get_host_cpu_refs()
                          if node.get_host_cpu_field(cpu_ref, 'number')
                              in cpus ]
        else:
            # pool not active, so it couldn't have any assigned cpus
            host_CPUs = []

        return host_CPUs

    def get_activated(self):
        """ Query if the pool is registered in XendStore.
            If pool uuid is not in XenStore, the pool is not activated.
            @return: True, if activated
            @rtype:  bool
        """
        return self.query_pool_id() != None

    def get_name_label(self):
        return self.name_label

    def get_name_description(self):
        return self.name_description

    def get_auto_power_on(self):
        return self.auto_power_on

    def get_ncpu(self):
        return self.ncpu

    def get_sched_policy(self):
        if len(self.sched_policy) == 0:
            # default scheduler selected
            return XendNode.instance().get_vcpus_policy()
        else:
            return self.sched_policy

    def get_proposed_CPUs(self):
        return [ str(cpu) for cpu in self.proposed_cpus ]

    def get_other_config(self):
        return self.other_config

    def set_name_label(self, name_label):
        self._checkName(name_label)
        self.name_label = name_label
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_name_description(self, name_descr):
        self.name_description = name_descr
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_auto_power_on(self, auto_power_on):
        self.auto_power_on = bool(int(auto_power_on))
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_ncpu(self, ncpu):
        _ncpu = int(ncpu)
        if _ncpu < 1:
            raise PoolError(XEND_ERROR_POOL_PARAM, 'ncpu')
        self.ncpu = _ncpu
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_sched_policy(self, sched_policy):
        if self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')
        self.sched_policy = sched_policy
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_proposed_CPUs(self, proposed_cpus):
        if self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')
        self.proposed_cpus = [ int(cpu) for cpu in proposed_cpus ]
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def set_other_config(self, other_config):
        self.other_config = other_config
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def destroy(self):
        """ In order to destroy a cpu pool, it must be deactivated """
        self.pool_lock.acquire()
        try:
            if self.get_activated():
                raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')
            XendBase.destroy(self)
        finally:
            self.pool_lock.release()
        XendNode.instance().save_cpu_pools()

    def activate(self):
        """ Create pool in hypervisor and add cpus.
            Preconditions:
            - pool not already active
            - enough unbound cpus available
            Actions:
            - create pool in hypervisor
            - select free cpus (preferred from proposed_CPUs list) and bind it to
              the pool
            - create entries in Xenstore
        """
        self.pool_lock.acquire()
        try:
            if self.get_activated():
                raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')
            sched_policy = self.get_sched_policy()
            if sched_policy not in XEN_SCHEDULER_TO_ID.keys():
                raise PoolError(XEND_ERROR_UNKOWN_SCHED_POLICY)
            unbound_cpus = set(self.unbound_cpus())
            if len(unbound_cpus) < self.ncpu:
                raise PoolError(XEND_ERROR_INSUFFICIENT_CPUS,
                                [str(self.ncpu), str(len(unbound_cpus))])

            # build list of cpu numbers to bind to pool
            cpu_set = set(self.proposed_cpus).intersection(unbound_cpus)
            if len(cpu_set) < self.ncpu:
                pool_cpus = (list(cpu_set) +
                             list(unbound_cpus.difference(cpu_set)))
            else:
                pool_cpus = list(cpu_set)
            pool_cpus = pool_cpus[0:self.ncpu]

            # create pool in hypervisor
            pool_id = xc.cpupool_create(
                sched = XEN_SCHEDULER_TO_ID.get(sched_policy, 0))

            self.update_XS(pool_id)
            # add cpus
            for cpu in pool_cpus:
                xc.cpupool_addcpu(pool_id, cpu)

        finally:
            self.pool_lock.release()

    def deactivate(self):
        """ Delete pool in hypervisor
            Preconditions:
            - pool is activated
            - no running VMs in pool
            Actions:
            - call hypervisor for deletion
            - remove path of pool in xenstore
        """
        self.pool_lock.acquire()
        try:
            if not self.get_activated():
                raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'deactivated')
            if len(self.get_started_VMs()) != 0:
                raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'in use')

            pool_id = self.query_pool_id()
            # remove cpus from pool
            cpus = []
            for pool_rec in xc.cpupool_getinfo():
                if pool_rec['cpupool'] == pool_id:
                    cpus = pool_rec['cpulist']
            for cpu_number in cpus:
                xc.cpupool_removecpu(pool_id, cpu_number)
            xc.cpupool_destroy(pool_id)

            # update XenStore
            xs_path = XS_POOLROOT + "%s/" % pool_id
            xstransact.Remove(xs_path)
        finally:
            self.pool_lock.release()

    def add_host_CPU_live(self, cpu_ref):
        """ Add cpu to pool, if it is currently not assigned to a pool.
            @param cpu_ref: reference of host_cpu instance to add
            @type  cpu_ref: str
        """
        if not self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'deactivated')
        node = XendNode.instance()
        number = node.get_host_cpu_field(cpu_ref, 'number')

        self.pool_lock.acquire()
        try:
            pool_id = self.query_pool_id()
            other_pool_ref = self.get_cpu_pool_by_cpu_ref(cpu_ref)
            if len(other_pool_ref) != 0:
                raise PoolError(XEND_ERROR_INVALID_CPU,
                            'cpu already assigned to pool "%s"' % other_pool_ref[0])
            xc.cpupool_addcpu(pool_id, number)
        finally:
            self.pool_lock.release()

        if number not in self.proposed_cpus:
            self.proposed_cpus.append(number)
        self._update_ncpu(pool_id)
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def remove_host_CPU_live(self, cpu_ref):
        """ Remove cpu from pool.
            After successfull call, the cpu is free.
            Remove of the last cpu of the pool is rejected.
            @param cpu_ref: reference of host_cpu instance to remove
            @type  cpu_ref: str
        """
        if not self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'deactivated')
        node = XendNode.instance()
        number = node.get_host_cpu_field(cpu_ref, 'number')

        self.pool_lock.acquire()
        try:
            pool_id = self.query_pool_id()
            pool_rec = {}
            for pool in xc.cpupool_getinfo():
                if pool['cpupool'] == pool_id:
                    pool_rec = pool
                    break

            if number in pool_rec['cpulist']:
                if len(pool_rec['cpulist']) < 2 and pool_rec['n_dom'] > 0:
                    raise PoolError(XEND_ERROR_LAST_CPU_NOT_REM,
                                    'could not remove last cpu')
                xc.cpupool_removecpu(pool_id, number)
            else:
                raise PoolError(XEND_ERROR_INVALID_CPU,
                                'CPU not assigned to pool')
        finally:
            self.pool_lock.release()

        if number in self.proposed_cpus:
            self.proposed_cpus.remove(number)
        self._update_ncpu(pool_id)
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def add_to_proposed_CPUs(self, cpu):
        if self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')

        _cpu = int(cpu)
        if _cpu not in self.proposed_cpus:
            self.proposed_cpus.append(_cpu)
            self.proposed_cpus.sort()
            if self._managed:
                XendNode.instance().save_cpu_pools()

    def remove_from_proposed_CPUs(self, cpu):
        if self.get_activated():
            raise PoolError(XEND_ERROR_BAD_POOL_STATE, 'activated')
        _cpu = int(cpu)
        if _cpu in self.proposed_cpus:
            self.proposed_cpus.remove(_cpu)
            if self._managed:
                XendNode.instance().save_cpu_pools()

    def add_to_other_config(self, key, value):
        self.other_config[key] = value
        if self._managed:
            XendNode.instance().save_cpu_pools()

    def remove_from_other_config(self, key):
        if key in self.other_config:
            del self.other_config[key]
        if self._managed:
            XendNode.instance().save_cpu_pools()


    #
    # Legacy RPC calls
    #
    def pool_new(cls, config):
        try:
            record = sxp2map(config)
            if record.has_key('proposed_CPUs') and \
               not isinstance(record['proposed_CPUs'], types.ListType):
                record['proposed_CPUs'] = [record['proposed_CPUs']]
            new_uuid = cls.create(record)
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())
        return new_uuid

    def pool_create(cls, config):
        try:
            record = sxp2map(config)
            if record.has_key('proposed_CPUs') and \
               not isinstance(record['proposed_CPUs'], types.ListType):
                record['proposed_CPUs'] = [record['proposed_CPUs']]
            new_uuid = genuuid.createString()
            pool = XendCPUPool(record, new_uuid, False)
            pool.activate()
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_start(cls, poolname):
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        try:
            pool.activate()
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_list(cls, names):
        sxprs = []
        try:
            node = XendNode.instance()
            xd = XendDomain.instance()
            pools = cls.get_all_records()
            for (pool_uuid, pool_vals) in pools.items():
                if pool_vals['name_label'] in names or len(names) == 0:
                    # conv host_cpu refs to cpu number
                    cpus = [ node.get_host_cpu_field(cpu_ref, 'number')
                             for cpu_ref in pool_vals['host_CPUs'] ]
                    cpus.sort()
                    pool_vals['host_CPU_numbers'] = cpus
                    # query VMs names. Take in account, that a VM
                    # returned by get_all_records could be destroy, now
                    vm_names = [ vm.getName()
                                 for vm in map(xd.get_vm_by_uuid,
                                               pool_vals['started_VMs'])
                                 if vm ]
                    pool_vals['started_VM_names'] = vm_names
                    pool_vals['auto_power_on'] = int(pool_vals['auto_power_on'])
                    sxprs += [[pool_uuid] + map2sxp(pool_vals)]
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())
        return sxprs

    def pool_destroy(cls, poolname):
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        try:
            pool.deactivate()
            if not pool.is_managed():
                pool.destroy()
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_delete(cls, poolname):
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        try:
            pool.destroy()
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_cpu_add(cls, poolname, cpu):
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        try:
            cpu_ref = cls._cpu_number_to_ref(int(cpu))
            if cpu_ref:
                pool.add_host_CPU_live(cpu_ref)
            else:
                raise PoolError(XEND_ERROR_INVALID_CPU,
                                'CPU unknown')
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_cpu_remove(cls, poolname, cpu):
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        try:
            cpu_ref = cls._cpu_number_to_ref(int(cpu))
            if cpu_ref:
                pool.remove_host_CPU_live(cpu_ref)
            else:
                raise PoolError(XEND_ERROR_INVALID_CPU,
                                'CPU unknown')
        except XendAPIError, ex:
            raise VmError(ex.get_api_error())

    def pool_migrate(cls, domname, poolname):
        dom = XendDomain.instance()
        pool = cls.lookup_pool(poolname)
        if not pool:
            raise VmError('unknown pool %s' % poolname)
        dominfo = dom.domain_lookup_nr(domname)
        if not dominfo:
            raise VmError('unknown domain %s' % domname)
        domid = dominfo.getDomid()
        if domid is not None:
            if domid == 0:
                raise VmError('could not move Domain-0')
            try:
                cls.move_domain(pool.get_uuid(), domid)
            except Exception, ex:
                raise VmError('could not move domain')
        dominfo.info['pool_name'] = poolname
        dom.managed_config_save(dominfo)

    pool_new        = classmethod(pool_new)
    pool_create     = classmethod(pool_create)
    pool_start      = classmethod(pool_start)
    pool_list       = classmethod(pool_list)
    pool_destroy    = classmethod(pool_destroy)
    pool_delete     = classmethod(pool_delete)
    pool_cpu_add    = classmethod(pool_cpu_add)
    pool_cpu_remove = classmethod(pool_cpu_remove)
    pool_migrate    = classmethod(pool_migrate)


    #
    # methods
    #

    def is_managed(self):
        """ Check, if pool is managed.
            @return: True, if managed
            @rtype: bool
        """
        return self._managed

    def query_pool_id(self):
        """ Get corresponding pool-id of pool instance from XenStore.
            @return: pool id or None
            @rytpe:  int
        """
        self.pool_lock.acquire()
        try:
            for pool_id in xstransact.List(XS_POOLROOT):
                uuid = xstransact.Read(XS_POOLROOT + "%s/" % pool_id, 'uuid')
                if uuid == self.get_uuid():
                    return int(pool_id)
        finally:
            self.pool_lock.release()

        return None

    def update_XS(self, pool_id):
        """ Write (or update) data in xenstore taken from instance.
            @param pool_id: Pool id to build path to pool data in xenstore
            @type  pool_id: int
        """
        self.pool_lock.acquire()
        try:
            xs_path = XS_POOLROOT + "%s/" % pool_id
            xs_entries = { 'uuid' : self.get_uuid(),
                           'name' : self.name_label,
                           'description' : self.name_description
                         }
            xstransact.Mkdir(xs_path)
            xstransact.Mkdir(xs_path, 'other_config')
            xstransact.Write(xs_path, xs_entries)
            xstransact.Write('%s%s' % (xs_path, 'other_config'),
                             self.other_config)
        finally:
            self.pool_lock.release()

    def _update_ncpu(self, pool_id):
        for pool_rec in xc.cpupool_getinfo():
            if pool_rec['cpupool'] == pool_id:
                self.ncpu = len(pool_rec['cpulist'])

    def _checkName(self, name):
        """ Check if a pool name is valid. Valid names contain alphabetic
            characters, digits, or characters in '_-.:/+'.
            The same name cannot be used for more than one pool at the same
            time.
            @param name: name
            @type name:  str
            @raise: PoolError if invalid
        """
        if name is None or name == '':
            raise PoolError(XEND_ERROR_POOL_PARAM, 'Missing Pool Name')
        if not re.search(r'^[A-Za-z0-9_\-\.\:\/\+]+$', name):
            raise PoolError(XEND_ERROR_POOL_PARAM, 'Invalid Pool Name')

        pool = self.lookup_pool(name)
        if pool and pool.get_uuid() != self.get_uuid():
            raise PoolError(XEND_ERROR_POOL_PARAM,
                'Pool name "%s" already exists' % name)


    #
    # class methods
    #

    def recreate_active_pools(cls):
        """ Read active pool config from hypervisor and create pool instances.
            - Query pool ids and assigned CPUs from hypervisor.
            - Query additional information for any pool from xenstore.
              If an entry for a pool id is missing in xenstore, it will be
              recreated with a new uuid and generic name (this is an error case)
            - Create an XendCPUPool instance for any pool id
            Function have to be called after recreation of managed pools.
        """
        log.debug('recreate_active_pools')

        for pool_rec in xc.cpupool_getinfo():
            pool = pool_rec['cpupool']

            # read pool data from xenstore
            path = XS_POOLROOT + "%s/" % pool
            uuid = xstransact.Read(path, 'uuid')
            if not uuid:
                # xenstore entry missing / invaild; create entry with new uuid
                uuid = genuuid.createString()
                name = "Pool-%s" % pool
                try:
                    inst = XendCPUPool( { 'name_label' : name }, uuid, False )
                    inst.update_XS(pool)
                except PoolError, ex:
                    # log error and skip domain
                    log.error('cannot recreate pool %s; skipping (reason: %s)' \
                        % (name, ex))
            else:
                (name, descr) = xstransact.Read(path, 'name', 'description')
                other_config = {}
                for key in xstransact.List(path + 'other_config'):
                    other_config[key] = xstransact.Read(
                        path + 'other_config/%s' % key)

                # check existance of pool instance
                inst = XendAPIStore.get(uuid, cls.getClass())
                if inst:
                    # update attributes of existing instance
                    inst.name_label = name
                    inst.name_description = descr
                    inst.other_config = other_config
                else:
                    # recreate instance
                    try:
                        inst = XendCPUPool(
                            { 'name_label' : name,
                              'name_description' : descr,
                              'other_config' : other_config,
                              'proposed_CPUs' : pool_rec['cpulist'],
                              'ncpu' : len(pool_rec['cpulist']),
                            },
                            uuid, False )
                    except PoolError, ex:
                        # log error and skip domain
                        log.error(
                            'cannot recreate pool %s; skipping (reason: %s)' \
                            % (name, ex))

    recreate_active_pools = classmethod(recreate_active_pools)


    def recreate(cls, record, current_uuid):
        """ Recreate a pool instance while xend restart.
            @param record: attributes of pool
            @type record:  dict
            @param current_uuid: uuid of pool to create
            @type current_uuid:  str
        """
        XendCPUPool(record, current_uuid)

    recreate = classmethod(recreate)


    def autostart_pools(cls):
        """ Start managed pools that are marked as autostart pools.
            Function is called after recreation of managed domains while
            xend restart.
        """
        cls.pool_lock.acquire()
        try:
            for inst in XendAPIStore.get_all(cls.getClass()):
                if inst.is_managed() and inst.auto_power_on and \
                   inst.query_pool_id() == None:
                    inst.activate()
        finally:
            cls.pool_lock.release()

    autostart_pools = classmethod(autostart_pools)


    def move_domain(cls, pool_ref, domid):
        cls.pool_lock.acquire()
        try:
            pool = XendAPIStore.get(pool_ref, cls.getClass())
            pool_id = pool.query_pool_id()

            xc.cpupool_movedomain(pool_id, domid)
        finally:
            cls.pool_lock.release()

    move_domain = classmethod(move_domain)


    def query_pool_ref(cls, pool_id):
        """ Get pool ref by pool id.
            Take the ref from xenstore.
            @param pool_id:
            @type  pool_id: int
            @return: ref
            @rtype:  str
        """
        uuid = xstransact.Read(XS_POOLROOT + "%s/" % pool_id, 'uuid')
        if uuid:
            return [uuid]
        else:
            return []

    query_pool_ref = classmethod(query_pool_ref)


    def lookup_pool(cls, id_or_name):
        """ Search XendCPUPool instance with given id_or_name.
            @param id_or_name: pool id or pool nameto search
            @type id_or_name:  [int, str]
            @return: instane or None if not found
            @rtype:  XendCPUPool
        """
        pool_uuid = None
        try:
            pool_id = int(id_or_name)
            # pool id given ?
            pool_uuid = cls.query_pool_ref(pool_id)
            if not pool_uuid:
                # not found -> search name
                pool_uuid = cls.get_by_name_label(id_or_name)
        except ValueError:
            # pool name given
            pool_uuid = cls.get_by_name_label(id_or_name)

        if len(pool_uuid) > 0:
            return XendAPIStore.get(pool_uuid[0], cls.getClass())
        else:
            return None

    lookup_pool = classmethod(lookup_pool)


    def number_of_pools(cls):
        return len(xc.cpupool_getinfo())

    number_of_pools = classmethod(number_of_pools)

    def _cpu_number_to_ref(cls, number):
        node = XendNode.instance()
        for cpu_ref in node.get_host_cpu_refs():
            if node.get_host_cpu_field(cpu_ref, 'number') == number:
                return cpu_ref
        return None

    _cpu_number_to_ref = classmethod(_cpu_number_to_ref)


    def unbound_cpus(cls):
        """ Build list containing the numbers of all cpus not bound to a pool.
            Info is taken from Hypervisor.
            @return: list of cpu numbers
            @rytpe:  list of int
        """
        return xc.cpupool_freeinfo()

    unbound_cpus = classmethod(unbound_cpus)




###########################################################
## Xen controller daemon
## Copyright (c) 2004, K A Fraser (University of Cambridge)
## Copyright (C) 2004, Mike Wray <mike.wray@hp.com>
## Copyright (C) 2005, XenSource Ltd
###########################################################

import os
import os.path
import signal
import stat
import sys
import threading
import time
import linecache
import pwd
import re
import traceback

import xen.lowlevel.xc

from xen.xend.XendLogging import log
from xen.xend import osdep
from xen.util import mkdir

import relocate
import udevevent
import SrvServer
from params import *


XEND_PROCESS_NAME = 'xend'


class Daemon:
    """The xend daemon.
    """
    def __init__(self):
        self.traceon = False
        self.tracefile = None
        self.traceindent = 0
        self.child = 0
        self.traceLock = threading.Lock()


    def cleanup_xend(self, kill):
        """Clean up the Xend pidfile.
        If a running process is found, kills it if 'kill' is true.

        @param kill: whether to kill the process
        @return running process id or 0
        """
        running = 0
        pid = read_pid(XEND_PID_FILE)
        if find_process(pid, XEND_PROCESS_NAME):
            if kill:
                os.kill(pid, signal.SIGTERM)
            else:
                running = pid
        if running == 0 and os.path.isfile(XEND_PID_FILE):
            os.remove(XEND_PID_FILE)
        return running


    def reloadConfig(self):
        """
        """
        pid = read_pid(XEND_PID_FILE)
        if find_process(pid, XEND_PROCESS_NAME):
            os.kill(pid, signal.SIGHUP)


    def status(self):
        """Returns the status of the xend daemon.
        The return value is defined by the LSB:
        0  Running
        3  Not running
        """
        if self.cleanup_xend(False) == 0:
            return 3
        else:
            return 0


    def fork_pid(self):
        """Fork and write the pid of the child to XEND_PID_FILE.

        @return: pid of child in parent, 0 in child
        """

        self.child = os.fork()

        if self.child:
            # Parent
            pidfile = open(XEND_PID_FILE, 'w')
            try:
                pidfile.write(str(self.child))
            finally:
                pidfile.close()

        return self.child


    def daemonize(self):
        # Detach from TTY.

        # Become the group leader (already a child process)
        os.setsid()

        # Fork, this allows the group leader to exit,
        # which means the child can never again regain control of the
        # terminal
        if os.fork():
            os._exit(0)

        # Detach from standard file descriptors, and redirect them to
        # /dev/null or the log as appropriate.
        # We open the log file first, so that we can diagnose a failure to do
        # so _before_ we close stderr.
        try:
            parent = os.path.dirname(XEND_DEBUG_LOG)
            mkdir.parents(parent, stat.S_IRWXU)
            fd = os.open(XEND_DEBUG_LOG, os.O_WRONLY|os.O_CREAT|os.O_APPEND, 0666)
        except Exception, exn:
            print >>sys.stderr, exn
            print >>sys.stderr, ("Xend failed to open %s.  Exiting!" %
                                 XEND_DEBUG_LOG)
            sys.exit(1)

        os.close(0)
        os.close(1)
        os.close(2)
        if XEND_DEBUG:
            os.open('/dev/null', os.O_RDONLY)
            os.dup(fd)
            os.dup(fd)
        else:
            os.open('/dev/null', os.O_RDWR)
            os.dup(0)
            os.dup(fd)
        os.close(fd)

        print >>sys.stderr, ("Xend started at %s." %
                             time.asctime(time.localtime()))

        
    def start(self, trace=0):
        """Attempts to start the daemons.
        The return value is defined by the LSB:
        0  Success
        4  Insufficient privileges
        """
        xend_pid = self.cleanup_xend(False)

        if self.set_user():
            return 4
        os.chdir("/")

        if xend_pid > 0:
            # Trying to run an already-running service is a success.
            return 0

        ret = 0

        # If we're not going to create a daemon, simply
        # call the run method right here.
        if not XEND_DAEMONIZE:
            self.tracing(trace)
            self.run(None)
            return ret
        
        # we use a pipe to communicate between the parent and the child process
        # this way we know when the child has actually initialized itself so
        # we can avoid a race condition during startup
        
        r,w = os.pipe()
        if os.fork():
            os.close(w)
            r = os.fdopen(r, 'r')
            try:
                s = r.read()
            finally:
                r.close()
            if not len(s):
                ret = 1
            else:
                ret = int(s)
        else:
            os.close(r)
            # Child
            self.daemonize()
            self.tracing(trace)

            # If Xend proper segfaults, then we want to restart it.  Thus,
            # we fork a child for running Xend itself, and if it segfaults
            # (or exits any way other than cleanly) then we run it again.
            # The first time through we want the server to write to the (r,w)
            # pipe created above, so that we do not exit until the server is
            # ready to receive requests.  All subsequent restarts we don't
            # want this behaviour, or the pipe will eventually fill up, so
            # we just pass None into run in subsequent cases (by clearing w
            # in the parent of the first fork).  On some operating systems,
            # restart is managed externally, so we won't fork, and just exit.
            while True:

                if not osdep.xend_autorestart:
                    self.run(os.fdopen(w, 'w'))
                    os._exit(0)

                pid = self.fork_pid()
                if pid:
                    if w is not None:
                        os.close(w)
                        w = None

                    (_, status) = os.waitpid(pid, 0)

                    if os.WIFEXITED(status):
                        code = os.WEXITSTATUS(status)
                        log.info('Xend exited with status %d.', code)
                        sys.exit(code)

                    if os.WIFSIGNALED(status):
                        sig = os.WTERMSIG(status)

                        if sig in (signal.SIGINT, signal.SIGTERM):
                            log.info('Xend stopped due to signal %d.', sig)
                            sys.exit(0)
                        else:
                            log.fatal(
                                'Xend died due to signal %d!  Restarting it.',
                                sig)
                else:
                    self.run(w and os.fdopen(w, 'w') or None)
                    # if we reach here, the child should quit.
                    os._exit(0)

        return ret

    def tracing(self, traceon):
        """Turn tracing on or off.

        @param traceon: tracing flag
        """
        if traceon == self.traceon:
            return
        self.traceon = traceon
        if traceon:
            self.tracefile = open(XEND_TRACE_FILE, 'w+', 1)
            self.traceindent = 0
            sys.settrace(self.trace)
            try:
                threading.settrace(self.trace) # Only in Python >= 2.3
            except:
                pass

    def print_trace(self, string):
        self.tracefile.write("%s: "% threading.currentThread().getName())
        for i in range(self.traceindent):
            ch = " "
            if (i % 5):
                ch = ' '
            else:
                ch = '|'
            self.tracefile.write(ch)
        self.tracefile.write(string)
            
    def trace(self, frame, event, arg):
        self.traceLock.acquire()
        try:
            if not self.traceon:
                print >>self.tracefile
                print >>self.tracefile, '-' * 20, 'TRACE OFF', '-' * 20
                self.tracefile.close()
                self.tracefile = None
                return None
            if event == 'call':
                code = frame.f_code
                filename = code.co_filename
                m = re.search('.*xend/(.*)', filename)
                if not m:
                    return None
                modulename = m.group(1)
                if modulename.endswith('.pyc'):
                    modulename = modulename[:-1]
                if modulename == 'sxp.py' or \
                   modulename == 'XendLogging.py' or \
                   modulename == 'XendMonitor.py' or \
                   modulename == 'server/SrvServer.py':
                    return None
                self.traceindent += 1
                self.print_trace("> %s:%s\n"
                                 % (modulename, code.co_name))
            elif event == 'line':
                filename = frame.f_code.co_filename
                lineno = frame.f_lineno
                self.print_trace("%4d %s" %
                                 (lineno, linecache.getline(filename, lineno)))
            elif event == 'return':
                code = frame.f_code
                filename = code.co_filename
                m = re.search('.*xend/(.*)', filename)
                if not m:
                    return None
                modulename = m.group(1)
                self.print_trace("< %s:%s\n"
                                 % (modulename, code.co_name))
                self.traceindent -= 1
            elif event == 'exception':
                self.print_trace("! Exception:\n")
                (ex, val, tb) = arg
                traceback.print_exception(ex, val, tb, 10, self.tracefile)
                #del tb
            return self.trace
        finally:
            self.traceLock.release()

    def set_user(self):
        # Set the UID.
        try:
            os.setuid(pwd.getpwnam(XEND_USER)[2])
            return 0
        except KeyError:
            print >>sys.stderr, "Error: no such user '%s'" % XEND_USER
            return 1

    def stop(self):
        return self.cleanup_xend(True)

    def run(self, status):
        try:
            log.info("Xend Daemon started")

            xc = xen.lowlevel.xc.xc()
            xinfo = xc.xeninfo()
            log.info("Xend changeset: %s.", xinfo['xen_changeset'])
            del xc

            relocate.listenRelocation()
            udevevent.listenUdevEvent()
            servers = SrvServer.create()
            servers.start(status)
            del servers
            
        except Exception, ex:
            print >>sys.stderr, 'Exception starting xend:', ex
            if XEND_DEBUG:
                traceback.print_exc()
            log.exception("Exception starting xend (%s)" % ex)
            if status:
                status.write('1')
                status.close()
            sys.exit(1)
            
def instance():
    global inst
    try:
        inst
    except:
        inst = Daemon()
    return inst


def read_pid(pidfile):
    """Read process id from a file.

    @param pidfile: file to read
    @return pid or 0
    """
    if os.path.isfile(pidfile) and os.path.getsize(pidfile):
        try:
            f = open(pidfile, 'r')
            try:
                return int(f.read())
            finally:
                f.close()
        except:
            return 0
    else:
        return 0


def find_process(pid, name):
    """Search for a process.

    @param pid: process id
    @param name: process name
    @return: pid if found, 0 otherwise
    """
    running = 0
    if pid:
        lines = os.popen('ps %d 2>/dev/null' % pid).readlines()
        exp = '^ *%d.+%s' % (pid, name)
        for line in lines:
            if re.search(exp, line):
                running = pid
                break
    return running


def main(argv = None):
    global XEND_DAEMONIZE
    
    XEND_DAEMONIZE = False
    if argv is None:
        argv = sys.argv

    try:
        daemon = instance()
    
        r,w = os.pipe()
        daemon.run(os.fdopen(w, 'w'))
        return 0
    except Exception, exn:
        log.fatal(exn)
        return 1


if __name__ == "__main__":
    sys.exit(main())



""" Reader-writer lock implementation based on a condition variable """

#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2008 International Business Machines Corp.
# Author: Stefan Berger <stefanb@us.ibm.com>
#============================================================================

from threading import Condition

class RWLock:

    RWLOCK_STATE_WRITER = -1
    RWLOCK_STATE_UNUSED = 0

    def __init__(self):
        self.__condition = Condition()
        self.__state = RWLock.RWLOCK_STATE_UNUSED
        self.__blocked_writers = 0

    def acquire_reader(self):
        self.__condition.acquire()
        while True:
            if self.__state == RWLock.RWLOCK_STATE_WRITER:
                self.__condition.wait()
            else:
                break
        self.__state += 1
        self.__condition.release()

    def acquire_writer(self):
        self.__condition.acquire()
        self.__acquire_writer(RWLock.RWLOCK_STATE_UNUSED)
        self.__condition.release()

    def __acquire_writer(self, wait_for_state):
        while True:
            if self.__state == wait_for_state:
                self.__state = RWLock.RWLOCK_STATE_WRITER
                break
            else:
                self.__blocked_writers += 1
                self.__condition.wait()
                self.__blocked_writers -= 1

    def release(self):
        self.__condition.acquire()
        if self.__state == RWLock.RWLOCK_STATE_WRITER:
            self.__state = RWLock.RWLOCK_STATE_UNUSED
        elif self.__state == RWLock.RWLOCK_STATE_UNUSED:
            assert False, 'Lock not in use.'
        else:
            self.__state -= 1
        self.__condition.notifyAll()
        self.__condition.release()


if __name__ == '__main__':
    from threading import Thread
    from time import sleep

    rwlock = RWLock()

    class Base(Thread):
        def __init__(self, name, timeout):
            self.name = name
            self.timeout = timeout
            Thread.__init__(self)

    class Reader(Base):
        def __init__(self, name = 'Reader', timeout = 10):
            Base.__init__(self, name, timeout)

        def run(self):
            print '%s begin' % self.name
            rwlock.acquire_reader()
            print '%s acquired' % self.name
            sleep(self.timeout)
            rwlock.release()
            print '%s end' % self.name

    class ReaderTwice(Base):
        def __init__(self, name = 'Reader', timeout = 10):
            Base.__init__(self, name, timeout)

        def run(self):
            print '%s begin' % self.name
            rwlock.acquire_reader()
            print '%s acquired once' % self.name
            sleep(self.timeout)
            rwlock.acquire_reader()
            print '%s acquired twice' % self.name
            sleep(self.timeout)
            rwlock.release()
            rwlock.release()
            print '%s end' % self.name

    class Writer(Base):
        def __init__(self, name = 'Writer', timeout = 10):
            Base.__init__(self, name, timeout)

        def run(self):
            print '%s begin' % self.name
            rwlock.acquire_writer()
            print '%s acquired' % self.name
            sleep(self.timeout)
            rwlock.release()
            print '%s end' % self.name

    def run_test(threadlist, msg):
        print msg
        for t in threadlist:
            t.start()
            sleep(1)
        for t in threads:
            t.join()
        print 'Done\n\n'

    threads = []
    threads.append( Reader('R1', 4) )
    threads.append( Reader('R2', 4) )
    threads.append( Writer('W1', 4) )
    threads.append( Reader('R3', 4) )
    run_test(threads,
             'Test: readers may bypass blocked writers')



#!/usr/bin/env python
#
# Copyright 2001-2002 by Vinay Sajip. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Vinay Sajip
# not be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# VINAY SAJIP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# VINAY SAJIP BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# This file is part of the Python logging distribution. See
# http://www.red-dove.com/python_logging.html
#
"""Test harness for the logging module. A test showing exception handling and use of SysLogHandler.

Copyright (C) 2001-2002 Vinay Sajip. All Rights Reserved.
"""
import sys, logging
import syslog

class SLHandler(logging.Handler):
    def __init__(self, ident, logopt=0, facility=syslog.LOG_USER):
        logging.Handler.__init__(self)
        self.ident = ident
        self.logopt = logopt
        self.facility = facility
        self.mappings = {
            logging.DEBUG: syslog.LOG_DEBUG,
            logging.INFO: syslog.LOG_INFO,
            logging.WARNING: syslog.LOG_WARNING,
            logging.ERROR: syslog.LOG_ERR,
            logging.CRITICAL: syslog.LOG_CRIT,
            }

    def encodeLevel(self, level):
        return self.mappings.get(level, syslog.LOG_INFO)

    def emit(self, record):
        syslog.openlog(self.ident, self.logopt, self.facility)
        msg = self.format(record)
        prio = self.encodeLevel(record.levelno)
        syslog.syslog(prio, msg)
        syslog.closelog()

def config():
    logging.basicConfig()
    logging.getLogger("").setLevel(logging.DEBUG)
    if __name__ == "__main__":
        fmt = logging.Formatter("%(asctime)s %(filename)s:%(lineno)d %(levelname)-5s - %(message)s")
        hdlr = logging.FileHandler("tmp.tmp")
        hdlr.setFormatter(fmt)
        logging.getLogger("").addHandler(hdlr)
    else:
        fmt = None
    hdlr = SLHandler("log_test1")
    if fmt:
        hdlr.setFormatter(fmt)
    logging.getLogger("").addHandler(hdlr)
    return hdlr

def run():
    logging.info("Starting...")
    try:
        print "7" + 4
    except Exception, e:
        logging.error("Problem %s (%d)", "ERROR", logging.ERROR, exc_info=1)
        logging.debug("Problem %s (%d)", "DEBUG", logging.DEBUG, exc_info=1)
    logging.info("Done.")


def main():
    hdlr = config()
    run()
    logging.getLogger("").removeHandler(hdlr)

if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Li Ge <lge@us.ibm.com>

# Test Description:
# Negative Test:
# Test for creating domains with same name. Verify fail.

import sys
import re

from XmTestLib import *

# Create a domain with name "default" (default XmTestDomain, with our ramdisk)
domain1 = XmTestDomain("default")

#start it
try:
    domain1.start(noConsole=True)
except DomainError, e:
    if verbose:
        print "Failed to create test domain1 because:"
        print e.extra
    FAIL(str(e))

# Create second domain with same name "default"
domain2 = XmTestDomain("default")

#start it
eyecatcher = "Pass"
try:
    domain2.start(noConsole=True)
except DomainError, e:
    eyecatcher = "Fail"
    # Stop the domain1 (nice shutdown)
    domain1.stop()

if eyecatcher != "Fail":
    domain2.stop()
    FAIL("xm create let me create a duplicate-named domain!") 



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2009 Fujitsu Technology Solutions
#============================================================================

""" Common function of cmds cpupool-new / cpupool-create.
"""

import sys
import types
import os

from xen.xend import PrettyPrint
from xen.xend import sxp

from xen.xm.opts import Opts, set_value, set_true, append_value, OptionError

GOPTS = Opts(use="""[options] [vars]

Create a cpupool.

Pool creation parameters can be set by command-line switches, from
a python configuration script or an SXP config file. See documentation
for --defconfig, --config. Configuration variables can be set using
VAR=VAL on the command line. For example name=Pool-1 sets name to Pool-1.

""")

GOPTS.opt('help', short='h',
          fn=set_true, default=0,
          use="Print this help.")

GOPTS.opt('help_config',
          fn=set_true, default=0,
          use="Print the available configuration variables (vars) for the "
          "configuration script.")

GOPTS.opt('path', val='PATH',
          fn=set_value, default='.:/etc/xen/cpupool',
          use="Search path for configuration scripts. "
          "The value of PATH is a colon-separated directory list.")

GOPTS.opt('defconfig', short='f', val='FILE',
          fn=set_value, default='xmdefconfig',
          use="Use the given Python configuration script."
          "The configuration script is loaded after arguments have been "
          "processed. Each command-line option sets a configuration "
          "variable named after its long option name, and these "
          "variables are placed in the environment of the script before "
          "it is loaded. Variables for options that may be repeated have "
          "list values. Other variables can be set using VAR=VAL on the "
          "command line. "
          "After the script is loaded, option values that were not set "
          "on the command line are replaced by the values set in the script.")

GOPTS.default('defconfig')

GOPTS.opt('config', short='F', val='FILE',
          fn=set_value, default=None,
          use="CPU pool configuration to use (SXP).\n"
          "SXP is the underlying configuration format used by Xen.\n"
          "SXP configurations can be hand-written or generated from Python "
          "configuration scripts, using the -n (dryrun) option to print "
          "the configuration.")

GOPTS.opt('dryrun', short='n',
          fn=set_true, default=0,
          use="Dry run - prints the resulting configuration in SXP but "
          "does not create the CPU pool.")

GOPTS.var('name', val='NAME', fn=set_value, default=None,
          use="CPU pool name.")

GOPTS.var('sched', val='SCHED', fn=set_value, default='credit',
          use="Scheduler to use for the CPU pool.")

GOPTS.var('cpus', val='CPUS', fn=set_value, default=1,
          use="CPUS to assign to the CPU pool.")

GOPTS.var('other_config', val='OTHER_CONFIG', fn=append_value, default=[],
          use="Additional info for CPU pool")


def sxp2map(sxp_val):
    record = {}
    for x in sxp_val:
        if isinstance(x, (types.ListType, types.TupleType)) \
           and len(x) > 1:
            if isinstance(x[1], (types.ListType, types.TupleType)):
                record[x[0]] = sxp2map(x[1])
            else:
                record[x[0]] = x[1]
    return record

def err(msg):
    print >> sys.stderr, "Error: %s" % msg
    sys.exit(-1)

def make_cpus_config(cfg_cpus):
    """ Taken from XendConfig. """
    # Convert 'cpus' to list of list of ints

    cpus_list = []
    # Convert the following string to list of ints.
    # The string supports a list of ranges (0-3),
    # seperated by commas, and negation (^1).
    # Precedence is settled by order of the string:
    #    "0-3,^1"      -> [0,2,3]
    #    "0-3,^1,1"    -> [0,1,2,3]
    def cnv(s):
        l = []
        for c in s.split(','):
            if c.find('-') != -1:
                (x, y) = c.split('-')
                for i in range(int(x), int(y)+1):
                    l.append(int(i))
            else:
                # remove this element from the list
                if len(c) > 0:
                    if c[0] == '^':
                        l = [x for x in l if x != int(c[1:])]
                    else:
                        l.append(int(c))
        return l

    if type(cfg_cpus) == list:
        if len(cfg_cpus) > 0 and type(cfg_cpus[0]) == list:
            # If sxp_cfg was created from config.sxp,
            # the form of 'cpus' is list of list of string.
            # Convert 'cpus' to list of list of ints.
            # Conversion examples:
            #    [['1']]               -> [[1]]
            #    [['0','2'],['1','3']] -> [[0,2],[1,3]]
            try:
                for c1 in cfg_cpus:
                    cpus = []
                    for c2 in c1:
                        cpus.append(int(c2))
                    cpus_list.append(cpus)
            except ValueError, e:
                raise err('cpus = %s: %s' % (cfg_cpus, e))
        else:
            # Conversion examples:
            #    ["1"]               -> [[1]]
            #    ["0,2","1,3"]       -> [[0,2],[1,3]]
            #    ["0-3,^1","1-4,^2"] -> [[0,2,3],[1,3,4]]
            try:
                for c in cfg_cpus:
                    cpus = cnv(c)
                    cpus_list.append(cpus)
            except ValueError, e:
                raise err('cpus = %s: %s' % (cfg_cpus, e))
    else:
        # Conversion examples:
        #  cpus=1:
        #    "1"      -> [[1]]
        #    "0-3,^1" -> [[0,2,3]]
        #  cpus=2:
        #    "1"      -> [[1],[1]]
        #    "0-3,^1" -> [[0,2,3],[0,2,3]]
        try:
            cpus_list = cnv(cfg_cpus)
        except ValueError, e:
            err('cpus = %s: %s' % (cfg_cpus, e))
    return cpus_list

def make_config(vals):
    config  = ['pool']
    config += [['name_label', vals.name]]
    config += [['sched_policy', vals.sched]]
    if type(vals.cpus) == int:
        config +=  [['ncpu', vals.cpus], ['proposed_CPUs' , []]]
    elif type(vals.cpus) == str and len(vals.cpus) > 1 and vals.cpus[0] == '#':
        try:
            config +=  [['ncpu', int(vals.cpus[1:])], ['proposed_CPUs' , []]]
        except ValueError, ex:
            err('Wrong illegal of parameter "cpus"')
    else:
        prop_cpus = make_cpus_config(vals.cpus)
        config +=  [['ncpu', len(prop_cpus)],
                    ['proposed_CPUs'] + prop_cpus]
    other_config = []
    for entry in vals.other_config:
        if '=' in entry:
            (var, val) = entry.strip().split('=', 1)
            other_config.append([var, val])
    config +=  [['other_config'] + other_config]
    return config

def parseCommandLine(argv):
    GOPTS.reset()
    args = GOPTS.parse(argv)

    if GOPTS.vals.help or GOPTS.vals.help_config:
        if GOPTS.vals.help_config:
            print GOPTS.val_usage()
        return (None, None)

    # Process remaining args as config variables.
    for arg in args:
        if '=' in arg:
            (var, val) = arg.strip().split('=', 1)
            GOPTS.setvar(var.strip(), val.strip())
    if GOPTS.vals.config:
        try:
            config = sxp.parse(file(GOPTS.vals.config))[0]
        except IOError, ex:
            raise OptionError("Cannot read file %s: %s" % (config, ex[1]))
    else:
        GOPTS.load_defconfig()
        if not GOPTS.getopt('name') and GOPTS.getopt('defconfig'):
            GOPTS.setopt('name', os.path.basename(
                GOPTS.getopt('defconfig')))
        config = make_config(GOPTS.vals)

    if GOPTS.vals.dryrun:
        PrettyPrint.prettyprint(config)
        return (None, None)

    return (GOPTS, config)

def help():
    return str(GOPTS)




#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Paul Larson  <pl@us.ibm.com>

# Description:
# Positive Tests:
# Test for xm shutdown
# 1) Create domain, verify it's up with console
# 2) shut down the domain, verify it's down

import time
import commands

from XmTestLib import *

# Create a domain (default XmTestDomain, with our ramdisk)
domain = XmTestDomain()

# Start it
try:
    console = domain.start()
except DomainError, e:
    if verbose:
        print "Failed to create test domain because:"
        print e.extra
    FAIL(str(e))

try:
    # Make sure a command succeeds
    run = console.runCmd("ls /bin")
except ConsoleError, e:
    FAIL(str(e))

# Close the console
domain.closeConsole()

# Stop the domain (nice shutdown)
status, output = traceCommand("xm shutdown %s" % domain.getName())
if status != 0:
    FAIL("good xm shutdown exited with bad %i != 0" % status)

# Verify the domain is gone
time.sleep(10)

if isDomainRunning(domain.getName()):

    traceCommand("xm list")
    if isDomainRunning(domain.getName()):
        FAIL("Guest domain failed to shutdown")
    else:
        FAIL("I had to run an xm list to update xend state!")





#!/usr/bin/env python
#
# Copyright 2001-2002 by Vinay Sajip. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Vinay Sajip
# not be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# VINAY SAJIP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# VINAY SAJIP BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# This file is part of the Python logging distribution. See
# http://www.red-dove.com/python_logging.html
#
"""
A test harness for the logging module. Tests MatchFilter.

Copyright (C) 2001-2002 Vinay Sajip. All Rights Reserved.
"""

import logging, types, string

class MatchFilter(logging.Filter):
    def __init__(self, **kwargs):
        self.dict = kwargs

    def matchOne(self, key, value, record):
        rv = getattr(record, key)
        if key != "name":
            if key not in ["message", "msg"]:
                return rv == value
            else:
                return string.find(str(rv), value) >= 0
        else:
            if rv == value:
                return 1
            nlen = len(value)
            if string.find(rv, value, 0, nlen) != 0:
                return 0
            if rv[nlen] == ".":
                return 1

    def matchValue(self, key, record):
        vl = self.dict [key]
        if type(vl) != types.ListType:
            rv = self.matchOne(key, vl, record)
        else:
            for v in vl:
                rv = self.matchOne(key, v, record)
                if rv:
                    break
        return rv

    def filter(self, record):
        rv = 1
        for k in self.dict.keys():
            if self.matchValue(k, record):
                rv = 0
                break
        return rv

def doLog(logger, n):
    logger.debug("Debug %d" % n)
    logger.info("Info %d" % n)
    logger.warning("Warning %d" % n)
    logger.error("Error %d" % n)
    logger.critical("Critical %d" % n)

def test():
    fmt = logging.Formatter("%(name)-10s %(levelname)-9s %(message)s")
    hand = logging.StreamHandler()
    hand.setFormatter(fmt)
    root = logging.getLogger("")
    root.setLevel(logging.DEBUG)
    root.addHandler(hand)
    loggers = ['A',
               'A.B',
               'A.BB',
               'A.C',
               'AA.B',
               'A.B.C',
               'A.B.C.D',
               'A.B.C.D.E',
               'Z.A.B',
              ]
    filt = MatchFilter(name = ['A.C', 'A.B.C'],                 #reject these loggers and their children
                       levelno = [logging.WARNING, logging.CRITICAL],  #reject these levels,
                       msg = 'bug 2'                            #reject if this in message
                      )
    hand.addFilter(filt)
    for log in loggers:
        doLog(logging.getLogger(log), loggers.index(log))

if __name__ == "__main__":
    test()
